/*
 * @brief emWin with touch example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "GUI.h"
#include "board.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* LCD width and height (from board LCD setup descriptor) */
#define LCD_WIDTH       BOARD_LCD.PPL
#define LCD_HEIGHT      BOARD_LCD.LPP

static bool touchInitialized = false;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/**
 * @brief GUI buffers required for emwin library
 */
#define GUI_BUF_ADDR  0x28100000
#define GUI_NUMBYTES  ((1024 * 1024) * 4)
U32 GUI_Memory_Size = GUI_NUMBYTES;
U32 GUI_Block_Size = 128;
U32 *GUI_Memory = (U32 *) GUI_BUF_ADDR;

/**
 * Systick 1mS clock required for emWin time functions
 */
volatile uint32_t systick_timems;

/**
 * Needed to convert touch coordinates to the format emWin uses
 */
extern bool display_Landscape;
extern bool display_MirrorY;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* mSec delay */
static void lcdDelay(uint32_t delay)
{
	delay += systick_timems;
	while (systick_timems < delay) {}
}

/* Initialize the LCD for the current board */
static void lcdInit(void)
{
	/* Board specific LCD pre-setup */
	Board_LCD_Init();

	/* Setup for current board */
	Chip_LCD_Init(LPC_LCD, (LCD_CONFIG_T *) &BOARD_LCD);
	Board_InitTouchController();
	Chip_LCD_SetUPFrameBuffer(LPC_LCD, (void *) FRAMEBUFFER_ADDR);
	Chip_LCD_PowerOn(LPC_LCD);
	lcdDelay(100);

	/* Turn on backlight */
	Board_SetLCDBacklight(1);
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief		SysTick handler sub-routine (1ms)
 * @return      Nothing
 */
void SysTick_Handler(void)						// SysTick Interrupt Handler @ 1000Hz
{
	static uint8_t ttick = 0;			// Touch Screen time tick
	int16_t tmp_x = -1, tmp_y = -1;
	systick_timems++;						// Increment 1ms time tick

	if (ttick++ == 20) {			// Touch Screen update period is 20ms
		ttick = 0;
		if (touchInitialized) {
			if (Board_GetTouchPos(&tmp_x, &tmp_y)) {
                // The way emWin works forces the touch coordinates to be 
                // transformed before they are handed over to emWin. This
                // transformation is based on the DISPLAY_ORIENTATION 
                // define in LCDConf.c, but is stored in the global display_* 
                // variables.
				if (display_Landscape) {
					if (display_MirrorY) {
						GUI_TOUCH_StoreState(LCD_HEIGHT - tmp_y, tmp_x);
					} else {
						GUI_TOUCH_StoreState(tmp_y, tmp_x);
					}
				} else {
					if (display_MirrorY) {
						GUI_TOUCH_StoreState(tmp_x, LCD_HEIGHT - tmp_y);
					} else {
						GUI_TOUCH_StoreState(tmp_x, tmp_y);
					}
				}
			}
		}
	}
}

extern void MainTask(void);

/**
 * @brief		Main function
 * @return      Nothing
 */
int main(void)
{
	SystemCoreClockUpdate();
	Board_Init();

	/* sysTick will handle touch events at 1KHz */
	SysTick_Config(Chip_Clock_GetRate(CLK_MX_MXCORE) / 1000);

	/* Setup LCD */
	lcdInit();
	touchInitialized = true;

	/* Main Task */
	MainTask();

	return 0;
}
