/*****************************************************************************
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#include "board.h"
#include "nand.h"
#include <string.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#define BUFF_SIZE  (2048 + 64)  /* 2048 page size + 64 byte spare */

static char nand_buff_before[BUFF_SIZE];
static char nand_buff[BUFF_SIZE];
static char nand_buff_rd[BUFF_SIZE];

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/


static void PrintGreeting(void)
{
	DEBUGOUT("\n*****************************************************\n");
	DEBUGOUT("*                                                   *\n");
	DEBUGOUT("* Demos NAND Flash on the LPC4357 OEM Board...      *\n");
	DEBUGOUT("* (C) Embedded Artists 2001-2013                    *\n");
	DEBUGOUT("*                                                   *\n");
	DEBUGOUT("*                             " __DATE__ ", " __TIME__ " *\n");
	DEBUGOUT("*****************************************************\n");
}

static void dumpRange(const uint8_t* data, int offset, int num)
{
	int i;
	for (i = 0; i < num; i += 16) {
		int x = i + offset;
		DEBUGOUT("0x%04x  %02x %02x %02x %02x %02x %02x %02x %02x  %02x %02x %02x %02x %02x %02x %02x %02x\n", x,
				 data[x], data[x+1], data[x+2], data[x+3], data[x+4], data[x+5], data[x+6], data[x+7],
				 data[x+8], data[x+9], data[x+10], data[x+11], data[x+12], data[x+13], data[x+14], data[x+15]);
	}
}

static void dump(const uint8_t* data, int offset)
{
	int off = offset;
	int num = 32; /* 32 bytes after offset */
	if (offset > 32) {
		off -= 32; /* to get some data before */
		num += 32; /* 32 before offset */
	}
	off -= off % 16; /* to get an even address */

	dumpRange(data, off, num);
}

static bool testNand(void)
{
  int blockNum = 100;
  int i = 0;
  uint32_t pageSize;

  if (!nand_init()) {
    DEBUGOUT("NAND: Unknown ID: 0x%08x\n", nand_readId());
    return false;
  }
  DEBUGOUT("NAND: found id 0x%08x\n", nand_readId());

  pageSize = nand_getPageSize();
  DEBUGOUT("NAND: Page Size = %d bytes\n", pageSize);
  DEBUGOUT("NAND: Block Size = %d bytes\n", nand_getBlockSize());
  DEBUGOUT("NAND: Redundant Size = %d bytes\n", nand_getRedundantSize());

  if (pageSize > BUFF_SIZE) {
	  DEBUGOUT("NAND: The page size requires larger test buffers.\n");
	  DEBUGOUT("NAND: Increase BUFF_SIZE from %d to %d and try again.\n", BUFF_SIZE, pageSize);
	  return false;
  }

  // find a valid block
  for (blockNum = 100; blockNum < NAND_NUM_BLOCKS; blockNum++) {
    if (!nand_isBlockValid(blockNum)) {
      continue;
    }

    break;
  }

  if (blockNum >= NAND_NUM_BLOCKS) {
    DEBUGOUT("NAND: Couldn't find valid NAND block\n");
    return FALSE;
  }
  DEBUGOUT("Using NAND block %d for these tests\n", blockNum);

  memset(nand_buff_before, 0, 2112);

  if (!nand_readPage(blockNum, 0, (uint8_t*) nand_buff_before)) {
	  DEBUGOUT("NAND: Failed to read from NAND\n");
    return FALSE;
  }

  for (i = 0; i < pageSize; i++) {
    nand_buff[i] = (char)(i+0);
  }

  if (!nand_eraseBlock(blockNum)) {
	  DEBUGOUT("NAND: Failed to erase block\n");
    return FALSE;
  }

  memset(nand_buff_rd, 0, BUFF_SIZE);

  if (!nand_readPage(blockNum, 0, (uint8_t*) nand_buff_rd)) {
	  DEBUGOUT("NAND: Failed to read from NAND\n");
    return FALSE;
  }

  for (i = 0; i < pageSize; i++) {
    if (nand_buff_rd[i] != 0xff) {
    	DEBUGOUT("NAND: Verification of erasing failed at offset %d (0x%x). Got 0x%02x, expected 0xff\n", i, i, nand_buff_rd[i]);
    	dump((uint8_t*)nand_buff_rd, i);
      return FALSE;
    }
  }

  if (!nand_writePage(blockNum, 0, (uint8_t*) nand_buff)) {
	  DEBUGOUT("NAND: Failed to write to NAND\n");
    return FALSE;
  }

  memset(nand_buff_rd, 0, BUFF_SIZE);

  if (!nand_readPage(blockNum, 0, (uint8_t*) nand_buff_rd)) {
	  DEBUGOUT("NAND: Failed to read from NAND\n");
    return FALSE;
  }

  for (i = 0; i < pageSize; i++) {
    if (nand_buff[i] != nand_buff_rd[i]) {
    	DEBUGOUT("NAND: Verify failed at offset %d (0x%x). Got 0x%02x, expected 0x%02x\n", i, i, nand_buff_rd[i], nand_buff[i]);
    	DEBUGOUT("Before:\n");
    	dump((uint8_t*)nand_buff_before, i);
    	DEBUGOUT("Wrote:\n");
    	dump((uint8_t*)nand_buff, i);
    	DEBUGOUT("Read:\n");
    	dump((uint8_t*)nand_buff_rd, i);
      return FALSE;
    }
  }

  return TRUE;
}


/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Main entry point
 * @return	Nothing
 */
int main(void)
{

	SystemCoreClockUpdate();
	Board_Init();

	/* Print the Demo Information */
	PrintGreeting();

	if (testNand()) {
		DEBUGOUT("\nAll tests PASSED!\n");
	} else {
		DEBUGOUT("\nOne or more tests FAILED!\n");
	}

	while(1) {
	}
}
