/*
 * @brief Color bar examples for LCDs using SWIM
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "lpc_swim.h"
#include "lpc_swim_font.h"
#include "lpc_rom8x16.h"
#include "lpc_winfreesystem14x16.h"
#include "lpc_x6x13.h"
#include <string.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* pointer to frame buffer */
static uint16_t *framebuffer = (uint16_t *) FRAMEBUFFER_ADDR;

#define FONT  font_x6x13
// #define FONT  font_rom8x16
// #define FONT  font_winfreesys14x16
#define DISPLAY_WIDTH  BOARD_LCD.PPL
#define DISPLAY_HEIGHT BOARD_LCD.LPP

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static void printCalibrationData(char* msg, const CALIBDATA_T* data)
{
	DEBUGSTR(msg);
	DEBUGOUT("{\r\n" \
			 "   Reference: { %d, %d }, { %d, %d }, { %d, %d }\r\n" \
			 "   Screen:    { %d, %d }, { %d, %d }, { %d, %d }\r\n" \
			 "}\r\n",
			 data->refX1, data->refY1, data->refX2, data->refY2, data->refX3, data->refY3,
			 data->scrX1, data->scrY1, data->scrX2, data->scrY2, data->scrX3, data->scrY3);
}

static void paintMessage(SWIM_WINDOW_T *win, char* pMsg)
{
  uint16_t msgLen = strlen((char *)pMsg);
  uint16_t x = 0;
  uint16_t y = 20;

  swim_set_pen_color(win, BLACK);
  swim_put_box(win, 0, 0, win->xpsize, 40);
  swim_set_pen_color(win, WHITE);

  // each character is 6 pixels wide
  x = (win->xpsize / 2) - (msgLen*6/2);

  swim_put_text_xy(win, (const CHAR*)pMsg, x, y);
}

static void drawCalibPoint(SWIM_WINDOW_T *win, uint16_t x, uint16_t y)
{
  swim_clear_screen(win, BLACK);
  paintMessage(win, "Calibrating, touch marked box...");

  swim_put_box(win, x-5, y-5, x+5, y+5);
}

static void calibrate(SWIM_WINDOW_T *win, bool isForced)
{
	CALIBDATA_T c;
	bool releaseNeeded = false;
	bool touchReleased = false;
	bool calibrated = false;
	int calibPoint = 0;
	int16_t x;
	int16_t y;

	if (Board_GetStoredCalibrationData(&c)) {
		printCalibrationData("Old data: ", &c);
		if (isForced) {
			DEBUGSTR("Found old calibration data but recalibration is requested.\r\n");
		} else {
			DEBUGSTR("The display is already calibrated!\r\n");
			DEBUGOUT("To force a recalibration, press the SW2 button while resetting\r\n");
			return;
		}
	} else {
		DEBUGSTR("Found no existing calibration data.\r\n");
	}

	DEBUGSTR("Starting Calibration...\r\n");
    DEBUGOUT("Follow instructions on screen\r\n");


    c.refX1 = 15;
    c.refY1 = win->ypsize - 15;
    c.refX2 = win->xpsize / 2;
    c.refY2 = 80;
    c.refX3 = win->xpsize - 15;
    c.refY3 = win->ypsize - 15;

    // Make sure the board is not adjusting the measured data
    Board_CalibrateTouch(NULL);

    drawCalibPoint(win, c.refX1, c.refY1);
    Board_GetTouchPos(&x, &y);

    while (!calibrated) {

      Board_DelayMs(100);

      if (!Board_GetTouchPos(&x, &y))
      {
        touchReleased = TRUE;
        continue;
      }

      if (releaseNeeded && !touchReleased)
        continue;

      touchReleased = FALSE;


      switch (calibPoint++)
      {
      case 0:
        c.scrX1 = x;
        c.scrY1 = y;
        drawCalibPoint(win, c.refX2, c.refY2);

        releaseNeeded = TRUE;
        break;
      case 1:
        c.scrX2 = x;
        c.scrY2 = y;
        drawCalibPoint(win, c.refX3, c.refY3);

        releaseNeeded = TRUE;
        break;
      case 2:
        c.scrX3 = x;
        c.scrY3 = y;
        paintMessage(win, "Calibrated!");
        calibrated = true;

        releaseNeeded = TRUE;
        Board_CalibrateTouch(&c);
		printCalibrationData("New data: ", &c);
		DEBUGOUT("Storing calibration data...");
		if (Board_StoreCalibrationData(&c)) {
			DEBUGOUT("DONE\r\n");
		} else {
			DEBUGOUT("FAILED!\r\n");
			DEBUGOUT("Board is still calibrated, but it could not be persistently stored!\r\n");
		}
        break;
      }
    }
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Main entry point
 * @return	Nothing
 */
int main(void)
{
	SWIM_WINDOW_T win;
	bool forced = false;
	int16_t x = 0;
	int16_t y = 0;

	SystemCoreClockUpdate();
	Board_Init();
	Board_Buttons_Init();

	if (Buttons_GetStatus() & BUTTONS_BUTTON2) {
		DEBUGOUT("Selected to do a forced calibration...\r\n");
		forced = true;
	}

	Board_LCD_Init();
	Board_InitTouchController();

	/* Initialize LCD, setup frame buffer, and enable vertical comp interrupt */
	Chip_LCD_Init(LPC_LCD, (LCD_CONFIG_T *) &BOARD_LCD);
	Chip_LCD_SetUPFrameBuffer(LPC_LCD, (void *) framebuffer);
	Chip_LCD_PowerOn(LPC_LCD);
	Chip_LCD_EnableInts(LPC_LCD, LCD_INTMSK_VCOMPIM);

	/* Turn on backlight */
	Board_SetLCDBacklight(1);

    swim_window_open(&win, DISPLAY_WIDTH, DISPLAY_HEIGHT, (COLOR_T*)FRAMEBUFFER_ADDR, 0, 0,
    		DISPLAY_WIDTH-1, DISPLAY_HEIGHT-1, 1, WHITE, BLACK, BLACK);

    DEBUGSTR("\r\n****************************************************\r\n");
    DEBUGSTR("*                                                  *\r\n");
    DEBUGSTR("* Touch calibration program for Embedded Artist's  *\r\n");
    DEBUGSTR("* display boards.                                  *\r\n");
    DEBUGSTR("*                                                  *\r\n");
    DEBUGSTR("****************************************************\r\n");

	calibrate(&win, forced);

	swim_clear_screen(&win, BLACK);
    swim_set_pen_color(&win, WHITE);
    swim_put_text_xy(&win, "Start drawing ", 10, 10);

	while (1) {
		if (Board_GetTouchPos(&x, &y)) {
			swim_put_pixel(&win, x, y);
		}
		Board_DelayMs(20);
	}
}
