#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_timer.h"

#include "lpc_sdmmc.h"
#include "lpc43xx_sdif.h"
#include "lpc43xx_sdmmc.h"

#include "../inc/diskio.h"

static int disk_inited;
static int bytes_transferred;
struct _mci_card_struct sdcardinfo;

static volatile int32_t sdio_wait_exit = 0;

volatile uint32_t error_origin = 0;
#define SAVE_LINE()  {error_origin = __LINE__; }

void timer_wait_us(void *t, volatile int us)
{
  TIM_Waitus(us);
}
void timer_wait_ms(void *t, volatile int ms)
{
  TIM_Waitms(ms);
}

DWORD get_fattime (void)
{
	return	  ((DWORD)(2012 - 1980) << 25)	/* Year = 2012 */
			| ((DWORD)1 << 21)				/* Month = 1 */
			| ((DWORD)1 << 16)				/* Day_m = 1*/
			| ((DWORD)0 << 11)				/* Hour = 0 */
			| ((DWORD)0 << 5)				/* Min = 0 */
			| ((DWORD)0 >> 1);				/* Sec = 0 */
}

/*******************************************************************************
* @brief		SDIO interrupt handler callback
* @param[in]    rinsts Optional input parameter
* @return		Return value is 0, not currently used
*******************************************************************************/
static uint32_t sdmmc_irq(uint32_t rinsts)
{
	/* Set wait exit flag to tell wait function we are ready. In an RTOS,
	   this would trigger wakeup of a thread waiting for the IRQ. */
	NVIC_DisableIRQ(SDIO_IRQn);
	sdio_wait_exit = 1;

	return 0;
}

/*********************************************************************//**
* @brief 		Sets up the SD event driven wakeup
* @param[in]	bits Status bits to poll for command completion
* @return 		None
**********************************************************************/
static void sdmmc_setup_wakeup(uint32_t bits)
{
    /* Wait for IRQ - for an RTOS, you would pend on an event here
	   with a IRQ based wakeup. */
	NVIC_ClearPendingIRQ(SDIO_IRQn);
	sdio_wait_exit = 0;
	LPC_SDMMC->INTMASK = bits;
	NVIC_EnableIRQ(SDIO_IRQn);
}

/*********************************************************************//**
* @brief 		A better wait callback for SDMMC driven by the IRQ flag
* @param[in]	bits Status bits to poll for command completion
* @return 		0 on success, or failure condition (-1)
**********************************************************************/
static uint32_t sdmmc_irq_driven_wait(uint32_t bits)
{
    uint32_t status;

	/* Wait for event, would be nice to have a timeout, but keep it
	   simple */
	while (sdio_wait_exit == 0);

	/* Get status and clear interrupts */
	status = LPC_SDMMC->RINTSTS;
	LPC_SDMMC->RINTSTS = status;
	LPC_SDMMC->INTMASK = 0;

	return status;
}

/*********************************************************************//**
* @brief 		Delay callback for timed SDIF/SDMMC functions
* @param[in]	time Number of milliSeconds to wait
* @return 		None
**********************************************************************/
void sdmmc_waitms(uint32_t time)
{
	/* In an RTOS, the thread would sleep allowing other threads to
	   run. For standalone operation, we just spin on a timer */
	TIM_Waitus(time * 1000);
}

#define SDFASTINOUTPIN (PUP_DISABLE | PDN_DISABLE | SLEWRATE_FAST | INBUF_ENABLE  | FILTER_DISABLE)  

 
DSTATUS disk_initialize(BYTE drv)
{
  uint32_t sdio_clk;
	if (drv) {
    SAVE_LINE();
		return STA_NOINIT;
  }
  
	scu_pinmux(0xc ,9 , PUP_DISABLE, FUNC7);           /* Pc.9 SDIO power */       //checked
	scu_pinmux(0xc ,8 , PUP_DISABLE | INBUF_ENABLE, FUNC7);  /* Pc.8 SDIO CD */					//checked
	scu_pinmux(0xc ,7 , SDFASTINOUTPIN, FUNC7);      /* Pc.7 SDIO D3 */       //checked
	scu_pinmux(0xc ,6 , SDFASTINOUTPIN, FUNC7);      /* Pc.6 SDIO D2 */				//checked
	scu_pinmux(0xc ,5 , SDFASTINOUTPIN, FUNC7);      /* Pc.5 SDIO D1 */				 //checked
	scu_pinmux(0xc ,4 , SDFASTINOUTPIN, FUNC7);      /* Pc.4 SDIO D0 */       //checked
	scu_pinmux(0xc ,0 , PUP_DISABLE | SLEWRATE_FAST, FUNC7);  /* Pc.0 SDIO clock */     //checked
	scu_pinmux(0xc ,10, SDFASTINOUTPIN, FUNC7);      /* Pc.10 SDIO command */ //checked
  
	CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_SDIO);

	/* The SDIO driver needs to know the SDIO clock rate */
	sdio_clk = CGU_GetPCLKFrequency(CGU_PERIPHERAL_SDIO);

	/* This init sdio with sdio_clk */
	sdif_init(sdio_clk, sdmmc_irq);

 	if (!sdif_card_ndetect()) {
    SAVE_LINE();
		return STA_NODISK;
  }

	sdif_power_onoff(1);
  
	/* Allow some time for the power supply to settle and the
	   card to fully seat in the slot */
  timer_wait_ms(NULL, 250);
  
	/* Enumerate the card once detected. Note this function may
	   block for a little while. */
	if (!sdmmc_acquire(sdmmc_setup_wakeup, sdmmc_irq_driven_wait,
		                 sdmmc_waitms, &sdcardinfo)) {
    SAVE_LINE();
		return STA_NODISK;
	}
  
	/* Setup card specific callbacks - use driver functions, but these can
	   be changed to custom functions or unique functions per slot. These
	   aren't used in the drivers, so setup of these is optional, but they
	   are setup here to be complete. */
	sdcardinfo.sdck_det = sdif_card_ndetect;
	sdcardinfo.sdck_wp = sdif_card_wp_on;
	sdcardinfo.sd_setpow = sdif_power_onoff;
	sdcardinfo.sd_setrst = sdif_reset;
  
  disk_inited = 1;
  return STA_OK;
}

DSTATUS disk_status(BYTE drv)
{
	if (drv || !disk_inited) {
    SAVE_LINE();
		return STA_NOINIT;
  }
  
	if (!sdif_card_ndetect()) {
    SAVE_LINE();
		return STA_NODISK;
  }
 	if (sdif_card_wp_on()) {
    SAVE_LINE();
		return STA_PROTECT;
  }

	return STA_OK;
}

DRESULT disk_read(
	BYTE drv,			/* Physical drive nmuber (0) */
	BYTE *buf,			/* Pointer to the data buffer to store read data */
	DWORD sector,		/* Start sector number (LBA) */
	BYTE count)			/* Sector count (1..128) */
{
	int stat = disk_status(drv);

	bytes_transferred = 0;
	if (stat != STA_OK && stat != STA_PROTECT) {
    SAVE_LINE();
		return RES_ERROR;
  }
	if (count == 0) {
    SAVE_LINE();
		return RES_PARERR;
  }
	bytes_transferred = sdmmc_read_blocks(buf, sector, sector + count - 1);
	if (bytes_transferred != count * MMC_SECTOR_SIZE) {
    SAVE_LINE();
		return RES_ERROR;
  }
	return RES_OK;
}

DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber (0) */
	const BYTE *buf,	/* Pointer to the data to be written */
	DWORD sector,		/* Start sector number (LBA) */
	BYTE count)			/* Sector count (1..128) */
{
	int stat = disk_status(drv);

	bytes_transferred = 0;
	if (stat != STA_OK) {
    SAVE_LINE();
		return RES_ERROR;
  }

	if (count == 0) {
    SAVE_LINE();
		return RES_PARERR;
  }
	bytes_transferred = sdmmc_write_blocks((BYTE *)buf, sector, sector + count - 1);
	if (bytes_transferred != count * MMC_SECTOR_SIZE) {
    SAVE_LINE();
		return RES_ERROR;
  }
  
	/* Poll card for program completion */
	while (sdmmc_get_state() != SDMMC_TRAN_ST);
  
	return RES_OK;
}

DRESULT disk_ioctl(
	BYTE drv,		/* Physical drive nmuber (0) */
	BYTE ctrl,		/* Control code */
	void *buf)		/* Buffer to send/receive control data */
{
	int stat = disk_status(drv);
	int size;

	if (stat != STA_OK && stat != STA_PROTECT) {
    SAVE_LINE();
		return RES_ERROR;
  }

	switch (ctrl)
	{
	case CTRL_SYNC:
		return RES_OK;

	case GET_SECTOR_COUNT:
		size = sdmmc_get_device_size();
	   	*(DWORD *)buf = size / MMC_SECTOR_SIZE;
		return RES_OK;

	case GET_SECTOR_SIZE:
		*(DWORD *)buf = MMC_SECTOR_SIZE;
		return RES_OK;

	case GET_BLOCK_SIZE:
		*(DWORD *)buf = 1;
		return RES_OK;

	default:
    SAVE_LINE();
		return RES_PARERR;
	}
}
