/*****************************************************************************
 *   eeprom.c:  Driver for the 24LC256 EEPROM on the OEM Board
 *
 *   Copyright(C) 2010, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

/*
 * NOTE: I2C must have been initialized before calling any functions in this
 * file.
 */

/******************************************************************************
 * Includes
 *****************************************************************************/


#include "lpc43xx.h"
#include "lpc43xx_i2c.h"
#include "string.h"
#include "stdio.h"
#include "uda1380_old.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

#define I2C_PORT (0)

#define UDA1380_I2C_ADDR    (0x1A)

#define UDA1380_REG_CLOCKS   (0x00)
#define UDA1380_REG_I2S      (0x01)
#define UDA1380_REG_PWR_CTRL (0x02)
#define UDA1380_REG_ANALOG   (0x03)
#define UDA1380_REG_HPHONE   (0x04)

#define UDA1380_REG_MASTER_VOL  (0x10)
#define UDA1380_REG_MIXER_VOL   (0x11)
#define UDA1380_REG_MODE_SEL    (0x12)
#define UDA1380_REG_MASTER_MUTE (0x13)
#define UDA1380_REG_MIXER_SET   (0x14)

#define UDA1380_REG_DEC_VOL     (0x20)
#define UDA1380_REG_PGA_SET     (0x21)
#define UDA1380_REG_ADC_SET     (0x22)
#define UDA1380_REG_AGC_SET     (0x23)

#define UDA1380_REG_RESET    (0x7F)


/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/


/******************************************************************************
 * Local Functions
 *****************************************************************************/

static Status I2CWrite(uint32_t addr, uint8_t* buf, uint32_t len) 
{
  I2C_M_SETUP_Type i2cData;

	i2cData.sl_addr7bit = addr;
	i2cData.tx_data = buf;
	i2cData.tx_length = len;
	i2cData.rx_data = NULL;
	i2cData.rx_length = 0;
	i2cData.retransmissions_max = 3;

  return I2C_MasterTransferData(LPC_I2C0, &i2cData, I2C_TRANSFER_POLLING);
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize the EEPROM Driver
 *
 *****************************************************************************/
void uda1380_init (void)
{
  uint8_t data[11];

  uda1380_reset();

  /*
   * Setup default values for the system settings 
   */

  /* evaluation modes and clock settings */
  data[0] = UDA1380_REG_CLOCKS;
  data[1] = 0x05; /* EN_DEC, EN_INT */
  data[2] = 0x08; //0x02; /* PLL1 */

  /* I2S input and output settings */
  data[3] = 0x00;
  data[4] = 0x10;//0x10;

  /* power control */
  data[5] = 0xA5;
  data[6] = 0xDF;

  /* analog mixer settings */
  data[7]  = 0x22;  /* left most volume attenuation */
  data[8]  = 0x22;  /* right most volume attenuation */

  I2CWrite((UDA1380_I2C_ADDR), data, 9);  

  /*
   * Interpolation filter
   */

  /* master volume control */
  data[0] = UDA1380_REG_MASTER_VOL;
  data[1] = 0x40; /* right channel volume attenuation */
  data[2] = 0x40; /* left channel volume attenuation */

  /* mixer volume control */
  data[3]  = 0x00;//0xff;	/* channel 2 volume attenuation */
  data[4]  = 0x00;	/* channel 1 volume attenuation */

  /* mode selection */
  data[5]  = 0x00;
  data[6]  = 0x00;

  /* master mute */
  data[7]  = 0x00;
  data[8]  = 0x00;

  data[9]  = 0x10;
  data[10]  = 0x00;
  
  I2CWrite((UDA1380_I2C_ADDR), data, 11);

  /*
   * Decimator
   */

  /* decimator volume control */
  data[0]  = UDA1380_REG_DEC_VOL;
  data[1]  = 0x00; /* ADC volume control left */
  data[2]  = 0x00; /* ADC volume control rigth */

  /* PGA settings and mute */
  data[3]  = 0x00;
  data[4]  = 0x00;

  /* ADC settings */
  data[5]  = 0x00;
  data[6]  = 0x0f;

  /* AGC settings */
  data[7]  = 0x00;
  data[8]  = 0x00;

  I2CWrite((UDA1380_I2C_ADDR), data, 9);


  /* evaluation modes and clock settings */
  data[0] = UDA1380_REG_CLOCKS;
  data[1] = 0x0F; /* EA-ADC, EN_DEC, EN_DAC, EN_INT */
  data[2] = 0x08;//0x02; /* PLL1 */
  I2CWrite((UDA1380_I2C_ADDR), data, 3);
}


void uda1380_reset(void)
{
  uint8_t data[3];

  data[0] = UDA1380_REG_RESET;
  data[1] = 0xff;
  data[2] = 0xff;

  I2CWrite((UDA1380_I2C_ADDR), data, 3);  
}
