/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "string.h"
#include "stdio.h"
#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_dac.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_i2c.h"
#include "lpc43xx_uart.h"
#include "lpc43xx_gpdma.h"

#include "acc.h"
#include "board.h"
#include "memreg.h"
#include "pca9532.h"
#include "ea_lcd_board.h"
#include "sdram.h"
#include "lpc_swim.h"
#include "lpc_swim_font.h"
#include "lpc_swim_image.h"
#include "truly_display.h"
#include "lodepng.h"
#include "cfgmem.h"
#include "lpc_heap.h"
#include "speaker.h"
#include "joystick.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define I2C_PORT (I2C_0)
#define MAIN_BUF_SZ 100

/******************************************************************************
 * Global variables
 *****************************************************************************/

extern const unsigned char sound_8k[];
extern int sound_sz;

extern const unsigned char demo_image[];
extern int demo_image_sz;

volatile int systick_counter = 0;

#define IMAGE_TO_SHOW  &demo_image[0]
#define IMAGE_SIZE     demo_image_sz

/******************************************************************************
 * Local variables
 *****************************************************************************/

static char main_buf[MAIN_BUF_SZ];
static uint16_t running_mode = 0;

static uint16_t sequence_0[] = {
    LED1, LED5, LED2, LED6, LED3, LED7, LED4,
    LED8, LED4, LED7, LED3, LED6, LED2, LED5,
};
static uint16_t sequence_1[] = {
    LED1|LED8, LED1|LED8, LED5|LED4, LED5|LED4,
    LED2|LED7, LED6|LED3, LED2|LED7, LED6|LED3,
    LED2|LED7, LED5|LED4, LED2|LED7, LED5|LED4,
};
static uint16_t sequence_2[] = {
    LED1|LED8, LED5|LED4, LED2|LED8, LED6|LED4, LED2|LED8, LED5|LED4,
    LED1|LED8, LED5|LED4, LED2|LED8, LED6|LED4, LED2|LED8, LED5|LED4,
};
static uint16_t sequence_3[] = {
    LED1|LED5|LED2|LED6, LED1|LED5|LED2|LED6,
    LED1|LED5|LED2|LED6, LED1|LED5|LED2|LED6,
    LED3|LED7|LED4|LED8, LED3|LED7|LED4|LED8,
    LED3|LED7|LED4|LED8, LED3|LED7|LED4|LED8,
};
static const uint16_t* const sequences[4] = {
    &sequence_0[0],
    &sequence_1[0],
    &sequence_2[0],
    &sequence_3[0],
};
static uint16_t sequence_lengths[4] = {14, 12, 12, 8};

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void greeting(void)
{
  console_sendString((uint8_t*)"\r\n***************************************************\r\n");
  console_sendString((uint8_t*)"*                                                 *\r\n");
  console_sendString((uint8_t*)"* Demo Application for the LPC4357 OEM Board...   *\r\n");
  console_sendString((uint8_t*)"* (C) Embedded Artists AB 2001-2013               *\r\n");
  console_sendString((uint8_t*)"*                                                 *\r\n");
  console_sendString((uint8_t*)"*                                                 *\r\n");
  console_sendString((uint8_t*)"*     Thank you for buying Embedded Artists'      *\r\n");
  console_sendString((uint8_t*)"*     LPC4357 Developer's Kit                     *\r\n");
  console_sendString((uint8_t*)"*                                                 *\r\n");
  console_sendString((uint8_t*)"*     ... and congratulations on your choice      *\r\n");
  console_sendString((uint8_t*)"*     of microcontroller!                         *\r\n");
  console_sendString((uint8_t*)"*                                                 *\r\n");
  console_sendString((uint8_t*)"***************************************************\r\n");
  console_sendString((uint8_t*)"Version: 1.4, (build " __DATE__ ")\r\n");

/*
 * Version History:
 *
 * 1.0  First version. 
 * 1.1  Updates mostly for the test part of this application which
 *      now runs at full speed. The demo still runs at 96MHz to allow
 *      it to run on unmodified boards. 
 * 1.2  Corrected NAND timing.
 * 1.3  Added RTC test.
 * 1.4  Changed EEPROM test to allow different EUI-48 identifiers.
 */
}

static void manipulateRunningLights(void)
{
  uint16_t state = 0;
  uint16_t new_mode = running_mode;
  static int pos = 0;

  state = (pca9532_getLedState(FALSE));

  if (state & KEY1) {
    new_mode = 0;
  } else if (state & KEY2) {
    new_mode = 1;
  } else if (state & KEY3) {
    new_mode = 2;
  } else if (state & KEY4) {
    new_mode = 3;
  } else {
    state = joystick_read();
    if (state & JOYSTICK_LEFT) {
      new_mode = (new_mode + 3) % 4; // select previous mode
    } else if (state & JOYSTICK_RIGHT) {
      new_mode = (new_mode + 1) % 4; // select next mode
    }
  }

  if (new_mode != running_mode) {
    running_mode = new_mode;
    pos = 0;
  }

  /* Update running light */
  pca9532_setLeds(sequences[running_mode][pos], (~sequences[running_mode][pos]&LED_MASK));

  pos++;
  if (pos >= sequence_lengths[running_mode]) {
    pos = 0;
  }
}

static void runLights(void)
{
  static Bool firstTime = TRUE;
  static int pos = 0;
  static uint16_t* memreg = (uint16_t*)MEMREG_BASE;
  static int32_t yoff;

  int8_t x;
  int8_t y;
  int8_t z;

  /* should be called every 100ms */
  if (firstTime) {
    memreg_init();
    acc_init();
    joystick_init();

    acc_read(&x, &y, &z);
    yoff = 0-y;
    firstTime = FALSE;
  }

  /* Update memory mapped LEDs based on accelerator position */
  if (pos&1) {
    acc_read(&x, &y, &z);
    y = y+yoff;
    if (y < 0) {
      *memreg = (1<<((y+75)/10)) << 0;
    } else {
      *memreg = (1<<(y/10)) << 8;
    }
  }

  /* Update running light */
  pos++;
  manipulateRunningLights();
}

// void SysTick_Handler(void)
// {
//   systick_counter++;
// }


/*
 * The decoded image produced by LodePNG is in 24 bit format but the display wants it in 16 bit
 * format (565). This function converts the image to 565 format.
 */
static void decodePNG(unsigned char** ppOut, unsigned* pW, unsigned* pH, const unsigned char* pIn, size_t insize)
{
  unsigned char* pTmp;
  unsigned error = LodePNG_decode24(&pTmp, pW, pH, pIn, insize);

  if (error == 0)
  {
    int x, y;
    uint16_t* pConverted;
    uint8_t r;
    uint8_t g;
    uint8_t b;
    int height = *pH;
    int width = *pW;
    int off = 0;

    *ppOut = lpc_new(width * height * 2);
    pConverted = (uint16_t*)*ppOut;

    for (y = 0; y < height; y++) {
      for (x = 0; x < width; x++) {
        r = pTmp[off    ];
        g = pTmp[off + 1];
        b = pTmp[off + 2];
        *pConverted = (((unsigned short)r & 0xF8) << 8) |
                       (((unsigned short)g & 0xFC) << 3) |
                       (((unsigned short)b & 0xF8) >> 3);
        pConverted++;
        off += 3;
      }
    }
    lpc_free(pTmp);
  }
  else
  {
    sprintf(main_buf, "Failed to decode PNG. Error %d\r\n", error);
    console_sendString((uint8_t*)main_buf);
  }
}

/**
 * Draw introduction text to the display
 */
static void drawIntro(SWIM_WINDOW_T* win1, int displayWidth, int displayHeight)
{
  SWIM_WINDOW_T win2;
  unsigned char* pOut = NULL;
  unsigned w, h;

  swim_clear_screen(win1, WHITE);
  swim_set_pen_color(win1, WHITE);

  decodePNG(&pOut, &w, &h, IMAGE_TO_SHOW, IMAGE_SIZE);

  if (displayWidth >= w && displayHeight >= h)
  {
    // Image is small enough to fit inside the physical boundaries of the
    // screen. Make sure it is centered
    swim_window_open(&win2, displayWidth, displayHeight, (COLOR_T*)(int32_t)(MEM_FB_BASE),
        (displayWidth-w)/2, (displayHeight-h)/2,
        (displayWidth-w)/2 + w - 1, (displayHeight-h)/2 + h - 1,
        0, WHITE, BLACK, BLACK);

    swim_put_image(&win2,(COLOR_T *)pOut,w,h);
  }
  else
  {
    // Image is small enough to fit inside the physical boundaries of the
    // screen but not in the correct orientation. Rotate it and make sure it is centered
    swim_window_open(&win2, displayWidth, displayHeight, (COLOR_T*)(int32_t)(MEM_FB_BASE),
        (displayWidth-h)/2, (displayHeight-w)/2,
        (displayWidth-h)/2 + h - 1, (displayHeight-w)/2 + w - 1,
        0, WHITE, BLACK, BLACK);

    swim_put_right_image(&win2,(COLOR_T *)pOut,w,h);
  }

  lpc_free(pOut);
}

static void playSimple(void)
{
  uint32_t cnt = 0;
  uint32_t off = 0;
  uint32_t sampleRate = 0;
  uint32_t delay = 0;
	DAC_CONVERTER_CFG_Type DAC_ConverterConfigStruct;

  DAC_Init(LPC_DAC);
	DAC_ConverterConfigStruct.DBLBUF_ENA = RESET;
	DAC_ConverterConfigStruct.CNT_ENA = RESET;
	DAC_ConverterConfigStruct.DMA_ENA = SET;
	DAC_ConfigDAConverterControl(LPC_DAC, &DAC_ConverterConfigStruct);
  CGU_EnableEntity(CGU_BASE_APB3, ENABLE);

  /* ChunkID */
  if (sound_8k[cnt] != 'R' && sound_8k[cnt+1] != 'I' &&
      sound_8k[cnt+2] != 'F' && sound_8k[cnt+3] != 'F')
  {
    console_sendString((uint8_t*)"Wrong format (RIFF)\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size*/
  cnt += 4;

  /* Format */
  if (sound_8k[cnt] != 'W' && sound_8k[cnt+1] != 'A' &&
      sound_8k[cnt+2] != 'V' && sound_8k[cnt+3] != 'E')
  {
    console_sendString((uint8_t*)"Wrong format (WAVE)\r\n");
    return;
  }
  cnt+=4;

  /* SubChunk1ID */
  if (sound_8k[cnt] != 'f' && sound_8k[cnt+1] != 'm' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != ' ')
  {
    console_sendString((uint8_t*)"Missing fmt\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size, audio format, num channels */
  cnt+= 8;

  sampleRate = (sound_8k[cnt] | (sound_8k[cnt+1] << 8) |
          (sound_8k[cnt+2] << 16) | (sound_8k[cnt+3] << 24));

  if (sampleRate != 8000) {
    console_sendString((uint8_t*)"Only 8kHz supported\r\n");
    return;
  }

  delay = 1000000 / sampleRate;

  cnt+=4;

  /* skip byte rate, align, bits per sample */
  cnt += 8;

  /* SubChunk2ID */
  if (sound_8k[cnt] != 'd' && sound_8k[cnt+1] != 'a' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != 'a')
  {
    console_sendString((uint8_t*)"Missing data\r\n");
    return;
  }
  cnt += 4;

  /* skip chunk size */
  cnt += 4;

  off = cnt;

//   while(1)
//   {
//     cnt = off;
    while(cnt++ < sound_sz)
    {
      DAC_UpdateValue(LPC_DAC, (sound_8k[cnt]&0x3FF));

      TIM_Waitus(delay);
    }
//   }
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int demo_main(void)
{
//  TIM_TIMERCFG_Type timerCfg;
  SWIM_WINDOW_T win1;
  int32_t dev_lcd = 0;
  int32_t framebuf = (int32_t)(MEM_FB_BASE);
  int32_t width = 0;
  int32_t height = 0;
  lcdb_result_t result;
  LCD_PARAM_T lp;
  int delay = 200;

  /* Initialize devices */

//   /* initialize timer */
//   TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
//   TIM_Init(LPC_TIM0, TIM_TIMER_MODE, &timerCfg);

//   console_init();
  greeting();

//   /* Initialize I2C */
//   I2C_Init(I2C_PORT, 100000);
//   I2C_Cmd(I2C_PORT, ENABLE);

//   /* Generate system tick at 10Hz */
//   SysTick_Config(SystemCoreClock/100);

  sdram_init();
  lpc_heap_init((void *)MEM_HEAP_BASE, MEM_HEAP_SIZE);

  do {
    if ((result = ea_lcdb_open(NULL, NULL, &dev_lcd)) == LCDB_RESULT_OK) {

      if ((result = ea_lcdb_getLcdParams(&lp)) != LCDB_RESULT_OK) {
        sprintf(main_buf, "ea_lcdb_getLcdParams FAILED (%d)\r\n", result);
        console_sendString((uint8_t*)main_buf);
        break;
      }

      width  = lp.pixels_per_line;
      height = lp.lines_per_panel;

      memset((void*)(framebuf), 0x00, width*height*2);

      if ((result = ea_lcdb_enable(dev_lcd, framebuf)) != LCDB_RESULT_OK) {
        sprintf(main_buf, "ea_lcdb_enable FAILED (%d)\r\n", result);
        console_sendString((uint8_t*)main_buf);
        break;
      }
    }
    else {
      /* Assuming that the unidentified display is a QVGA display */
      width  = 240;
      height = 320;
      memset((void*)(framebuf), 0x00, width*height*2);

      truly_lcd_open(framebuf);
    }

    /* Create a SWIM window */
    swim_window_open(&win1, width, height, (COLOR_T*)framebuf, 0, 0,
      width-1, height-1, 1, WHITE, BLACK, BLACK);

    drawIntro(&win1, width, height);

  } while(0);

#if 0
  initSpeaker();
  playSound();
#else
  playSimple();
#endif

  while (1) {
    /* Wait forever */
    if (systick_counter > 100) {
      uint16_t state = joystick_read();
      if (state & JOYSTICK_UP) {
        delay = 50;
      } else if (state & JOYSTICK_CENTER) {
        delay = 200;
      } else if (state & JOYSTICK_DOWN) {
        delay = 400;
      }
    }
    if (systick_counter > delay) {
      runLights();
      systick_counter = 0;
    }
  }
}

