/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lpc_types.h"
#include "lpc43xx_adc.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_i2c.h"
#include "lpc43xx_i2s.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_rtc.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_ssp.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_uart.h"
#include "spifi_rom_api.h"

#include "sdram.h"
#include "oem_eeprom.h"
#include "nand.h"

#include "norflash.h"
#include "ff.h"
#include "lm75a.h"
#include "acc.h"
#include "joystick.h"
#include "pca9532.h"
#include "memreg.h"
#include "uda1380.h"

#include "ethapi.h"

#include "truly_display.h"
#include "lcd_driver.h"
#include "lpc_swim.h"
#include "lpc_swim_font.h"
#include "lpc_x5x7.h"

#include "board.h"
#include "usb.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define I2C_PORT (LPC_I2C0)

#define LOG_INFO(x)  console_sendString((uint8_t*)x)
#define LOG_DEBUG(x) console_sendString((uint8_t*)x)

#define FRAMEBUF_SZ  (240*320*2)

#define BACK_COLOR LIGHTGRAY

/* [31:24] Implementer code, 0x41 = ARM
 * [23:20] Variant number, r in rnpn, 0x0 = r0p0
 * [19:16] Constant 0xF
 * [15:4]  PartNo, 0xC24 = Cortex-M4
 * [3:0]   Revision number, p in rnpn, 0x1 = r0p1
 *
 * So a LPC4357 is ARM Cortex-M4, variant 0, revision 1
 */
#define CPUID_IS_4357  ((*((uint32_t*)0xE000ED00)) == 0x410FC241)


/******************************************************************************
 * Local variables
 *****************************************************************************/

/* UART buffer */
static char buff[512];

static uint16_t  logy = 5;
static uint32_t lcd_ok = FALSE;
static SWIM_WINDOW_T* win;

static COLOR_T inst_colors[] = {WHITE, YELLOW};
static int curr_instr_color = 0;

/* For SDCARD tests */
static FATFS lFatfs[1];

/* To keep track of test duration */
static uint32_t testStartTime;
static volatile uint32_t mstick = 0;



/******************************************************************************
 * Global variables
 *****************************************************************************/

extern volatile int systick_counter;

/******************************************************************************
 * External Functions
 *****************************************************************************/

void calibrateStart(SWIM_WINDOW_T *win);
int demo_main(void);

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n***************************************************\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("* Test program part 2 for LPC4357 OEM Boards...   *\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                  *\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("*                           " __DATE__ ", " __TIME__ " *\r\n");
  console_sendString("***************************************************\r\n");
}

static COLOR_T next_color(void)
{
  COLOR_T c = inst_colors[curr_instr_color++];

  curr_instr_color = curr_instr_color % (sizeof(inst_colors)/sizeof(COLOR_T));
  
  return c;  
}

static void logResult(char *msg, uint32_t result)
{
  sprintf(buff, "%9s: %s  (%dms)", msg, (result ? "OK    " : "FAILED"), (mstick-testStartTime));
  LOG_INFO(buff); 
  LOG_INFO("\r\n");
  
  buff[17] = '\0'; //to avoid printing the timing in the display

  if (lcd_ok) {
    swim_set_bkg_color(win, BACK_COLOR);
    swim_put_text_xy(win, (const CHAR*)buff, 10, logy);
    logy += 10; 
  }
}

static void logInst(char* msg, int line, uint32_t color)
{
  COLOR_T penc;

  LOG_INFO(msg);   
  LOG_INFO("\r\n");

  if (lcd_ok) {
    penc = win->pen;

    swim_set_bkg_color(win, color);
    swim_set_pen_color(win, BLACK);

    if (line == 0) {
      swim_set_fill_color(win, color);
      swim_put_box(win, 0, 250, DISPLAY_WIDTH, DISPLAY_HEIGHT);
       
    }

    swim_put_text_xy(win, (const CHAR*)msg, 5, 250+5+line*8);
    swim_set_bkg_color(win, BACK_COLOR);
    swim_set_pen_color(win, penc);
  }
}

static void logInstClear(void)
{
  if (lcd_ok) {
    swim_set_fill_color(win, BACK_COLOR);
    swim_put_box(win, 0, 250, DISPLAY_WIDTH, DISPLAY_HEIGHT);
  }
}

static int logPos = 0;
#define LOG_INFO_Y      5
#define LOG_INFO_HEIGHT 60
#define LOG_INFO_COLOR  WHITE

static void logInfoStart(void)
{

  logPos = 0;

  if (lcd_ok) {
    swim_set_fill_color(win, LOG_INFO_COLOR);
    swim_put_box(win, DISPLAY_WIDTH/2, LOG_INFO_Y, DISPLAY_WIDTH-1, 
      LOG_INFO_HEIGHT);
  }

}

static void logInfo(char* msg)
{
  COLOR_T penc;
  LOG_INFO(msg);   
  LOG_INFO("\r\n");  

  if (lcd_ok) {

    penc = win->pen;

    swim_set_bkg_color(win, LOG_INFO_COLOR);
    swim_set_pen_color(win, BLACK);

    swim_put_text_xy(win, (const CHAR*)msg, DISPLAY_WIDTH/2+5, LOG_INFO_Y+logPos*8);

    swim_set_bkg_color(win, BACK_COLOR);
    swim_set_pen_color(win, penc);
  }

  logPos++;
}

static void logInfoStop(void)
{

  if (lcd_ok) {
    swim_set_fill_color(win, BACK_COLOR);
    swim_put_box(win, DISPLAY_WIDTH/2, LOG_INFO_Y, DISPLAY_WIDTH-1, 
      LOG_INFO_HEIGHT);
  }

}


void SysTick_Handler(void)
{
//  disk_timerproc();
  mstick++;
  systick_counter++;
}


static uint32_t test_sdram(void)
{
  uint32_t result = TRUE; 
  volatile uint32_t *wr_ptr; 
  uint32_t lfsr;

  int i = 0;

  testStartTime = mstick;
  
  if (!sdram_init()) {
    return FALSE;
  }


  wr_ptr = (uint32_t *)SDRAM_BASE;

  // clear content (only testing half of the SDRAM size)
  for (i = 0; i < SDRAM_SIZE/4/2; i++)
  {
    *wr_ptr = 0;
    *wr_ptr = 0xFFFFFFFF; //Toggle

    *wr_ptr++ = 0;  //Reset
  }

  wr_ptr = (uint32_t *)SDRAM_BASE;
  lfsr = 1;
  for (i = 0; i < (SDRAM_SIZE/4/2); i++) //4 bytes per word
  {
    *wr_ptr++ = lfsr;	  
    lfsr = (lfsr >> 1) ^ (unsigned int)(0 - (lfsr & 1u) & 0xd0000001u);
  }


  //Verify
  wr_ptr = (uint32_t *)SDRAM_BASE;
  lfsr = 1;
  for (i = 0; i < (SDRAM_SIZE/4/2); i++) //4 bytes per word
  {
    if (*wr_ptr++ != lfsr)
    {
      result = FALSE;
      break;
    }
    lfsr = (lfsr >> 1) ^ (unsigned int)(0 - (lfsr & 1u) & 0xd0000001u); 
  }
  
  return result;
}

static uint32_t test_rtc(void)
{
  uint32_t result = TRUE;
  uint32_t timeout;

  RTC_TIME_Type time = {
    0,    /*!< Seconds Register */
    48,   /*!< Minutes Register */
    9,    /*!< Hours Register */
    22,   /*!< Day of Month Register */
    1,    /*!< Day of Week Register */
    128,  /*!< Day of Year Register */
    4,    /*!< Months Register */
    2014, /*!< Years Register */
  };

  LOG_INFO("RTC: Initializing RTC, will take a couple of seconds...\r\n");

  RTC_Init(LPC_RTC);
  RTC_SetFullTime(LPC_RTC, &time);
  RTC_Cmd(LPC_RTC, ENABLE);

  LOG_DEBUG("RTC: Time set to 0 seconds, waiting for it to reach 2 seconds...\r\n");
  timeout = mstick + 4000;
  while((LPC_RTC->SEC != 2) && (mstick < timeout)) {
  }
  if (mstick >= timeout) {
    // timed out
    LOG_DEBUG("RTC: Timeout. Crystal problems?\r\n");
  }
  
  return result;
}

#define E_WRITE_LEN  128//(OEM_EEPROM0_TOTAL_SIZE)   //200
#define E_WRITE_OFF   0                         //240
#define E_READ_LEN   128
#define E_B(__x)  (b[E_READ_OFF + (__x)
static uint32_t test_eepromOem(void) 
{
  uint8_t b[E_WRITE_LEN];
  int16_t len = 0;
//  uint16_t offset = E_WRITE_OFF;
  int i = 0;

  testStartTime = mstick;

  oem_eeprom_init();

  memset(b, 0, 6);

  /* Verify the EUI-48 data */
  len = oem_eeprom_readUniqueID(OEM_EEPROM_2K, b);
  if (len != 6) {
    LOG_DEBUG("EEPROM: Failed to read all data\r\n");
    return FALSE;
  }

  if ((b[0] != 0x00) || (b[1] != 0x04) || (b[2] != 0xA3))
  {
#define IS_INVALID(__b) (((__b) == 0x00) || ((__b) == 0xff))
    if (IS_INVALID(b[0]) && IS_INVALID(b[1]) && IS_INVALID(b[2])) {
      sprintf(buff, "EEPROM: Invalid EUI-48: %02x%02x%02x, expected 0004A3\r\n", b[0], b[1], b[2]);
      LOG_DEBUG(buff);
      return FALSE;
    } else {
      /* The read bytes are not all 0x00 or 0xff so it could be a new valid id, accept it */
      sprintf(buff, "EEPROM: New EUI-48: %02x%02x%02x, expected 0004A3\r\n", b[0], b[1], b[2]);
    }
  }
  
  sprintf(buff, "EEPROM: Found MAC %02x-%02x-%02x-%02x-%02x-%02x\r\n", 
          b[0], b[1], b[2], b[3], b[4], b[5]);
  LOG_DEBUG(buff);

  for (i = 0; i < E_WRITE_LEN; i++) {
    b[i] = (uint8_t)(i+4);
  }

  len = oem_eeprom_write(OEM_EEPROM_2K, b, E_WRITE_OFF, E_WRITE_LEN);
  if (len != E_WRITE_LEN) {
    LOG_DEBUG("EEPROM: Failed to write data\r\n");
    return FALSE;
  }

  memset(b, 0, E_WRITE_LEN);

  len = oem_eeprom_read(OEM_EEPROM_2K, b, E_WRITE_OFF, E_WRITE_LEN);
  if (len != E_WRITE_LEN) {
    LOG_DEBUG("EEPROM: Failed to read all data\r\n");
    return FALSE;
  }

  for (i = 0; i < E_WRITE_LEN; i++) {
    if (b[i] != (uint8_t)(i+4)) {
      LOG_DEBUG("EEPROM: Verification failed\r\n");

      return FALSE;
    }
  }

  return TRUE;
}



static char nand_buff[2112];
static uint32_t test_nand(void) 
{
  int blockNum = 100;
  int i = 0;

  testStartTime = mstick;

  if (!nand_init()) {
    uint32_t id = nand_readId();
    sprintf(buff, "NAND: Unknown ID: 0x%08x\r\n", nand_readId());
    LOG_DEBUG(buff);
    return FALSE;
  }

  // find a valid block
  for (blockNum = 100; blockNum < NAND_NUM_BLOCKS; blockNum++) {
    if (!nand_isBlockValid(blockNum)) {
      continue;
    }

    break;
  }

  if (blockNum >= NAND_NUM_BLOCKS) {
    LOG_DEBUG("NAND: Couldn't find valid NAND block\r\n");
    return FALSE;  
  }

  for (i = 0; i < nand_getPageSize(); i++) {
    nand_buff[i] = (char)(i+0);
  }

  if (!nand_eraseBlock(blockNum)) {
    LOG_DEBUG("NAND: Failed to erase block\r\n");
    return FALSE;        
  }

  if (!nand_writePage(blockNum, 0, (uint8_t*) nand_buff)) {
    LOG_DEBUG("NAND: Failed to write to NAND\r\n");        
    return FALSE;
  }

  memset(nand_buff, 0, 2112);

  if (!nand_readPage(blockNum, 0, (uint8_t*) nand_buff)) {
    LOG_DEBUG("NAND: Failed to read from NAND\r\n");        
    return FALSE;
  }

  for (i = 0; i < nand_getPageSize(); i++) {
    if (nand_buff[i] != (char)(i+0)) {
      LOG_DEBUG("NAND: Verify failed\r\n");        
      return FALSE;
    }
  }

  return TRUE;
}


static uint32_t test_nor(void) 
{
  testStartTime = mstick;

  return norflash_init();
}

static uint32_t test_sdcard(void) 
{
  FRESULT res;
  FIL file;
  UINT numRead = 0;
  uint32_t result = TRUE;
  int i = 0;

  testStartTime = mstick;

  res = f_mount(0, &lFatfs[0]);
  if (res != FR_OK) {
    LOG_DEBUG("SDCARD: Failed to mount FS\r\n");
    return FALSE;
  }

  res = f_open (&file, "testfile.txt", FA_READ);
  if (res) {
    LOG_DEBUG("SDCARD: Failed to open testfile.txt\r\n");
    return FALSE;
  }


  res =  f_read (&file, nand_buff, 100, &numRead);
  if (res || numRead <= 0) {
    LOG_DEBUG("SDCARD: Failed to read file\r\n");
    result = FALSE;
  }

  for ( i = 0; result == TRUE && i < 100; i ++)
  {
    if (nand_buff[i] != ('0' + (i%10)))
    {
      LOG_DEBUG("SDCARD: Failed to read file\r\n");
      result = FALSE;
      break;
    }
  }

  res =  f_close (&file);
  if (res) {
    LOG_DEBUG("SDCARD: Failed to close ip.txt\r\n");
    return FALSE;
  }

  return result;
}

static uint32_t test_temp(void) 
{
#define TEMP_MIN 1700
#define TEMP_MAX 3000

  int32_t temp;

  testStartTime = mstick;

  lm75a_init();

  temp = lm75a_readTemp();
  temp = lm75a_readTemp();
  temp = lm75a_readTemp();
  temp = lm75a_readTemp();
  temp = lm75a_readTemp();
  temp = lm75a_readTemp();

  if (temp <= TEMP_MIN || temp >= TEMP_MAX) {
      sprintf(buff, "TEMP: out of range temp=%d", temp);
      LOG_DEBUG(buff);
  }

  return (temp > TEMP_MIN && temp < TEMP_MAX);
}

static uint32_t test_acc(void) 
{
#if 0
#define ACC_TOTAL_TIME_MS 20000
#define ACC_DELAY_MS 250
#define MIN_X_OK 40
#define MIN_Y_OK 40
#define MIN_Z_OK 30

  uint32_t time = 0;
  uint32_t lasttime = 0;

  int32_t xoff;
  int32_t yoff;
  int32_t zoff;

  int8_t x;
  int8_t y;
  int8_t z;

  int8_t minx=100;
  int8_t miny=100;
  int8_t minz=100;

  int8_t maxx=-100;
  int8_t maxy=-100;
  int8_t maxz=-100;

  uint32_t xdone = FALSE;
  uint32_t ydone = FALSE;
  uint32_t zdone = FALSE;

  COLOR_T color = next_color();

  acc_init();  

  acc_read(&x, &y, &z);
  xoff = 0-x;
  yoff = 0-y;
  zoff = 64-z;

  logInst("ACC Test", 0, color);
  logInst("  Tilt board left, right, up", 1, color);
  logInst("  and down to test accelerometer", 2, color);

  sprintf(buff, "  %d sec remaining", (ACC_TOTAL_TIME_MS-time)/1000);
  logInst(buff, 5, color);

  logInfoStart();
  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);

  while(1) {
  
    acc_read(&x, &y, &z);
    x = x+xoff;
    y = y+yoff;
    z = z+zoff;

    if (x < minx)
      minx = x;

    if(x > maxx)
      maxx = x;

    if (y < miny)
      miny = y;

    if(y > maxy)
      maxy = y;

    if (z < minz)
      minz = z;

    if(z > maxz)
      maxz = z;

    /* check if values have changed with the minimum accepted value */
  
    if(!xdone && (maxx-minx) > MIN_X_OK){
      xdone = TRUE;
      logInfo("Up-down: OK");
    }

    if(!ydone && (maxy-miny) > MIN_Y_OK) {
      ydone = TRUE;
      logInfo("Left-right   : OK");
    }

    if(!zdone && (maxz-minz) > MIN_Z_OK) {
      zdone = TRUE;
      logInfo("Z-axis    : OK");
    }

    if (xdone && ydone/* && zdone*/)
      break;  

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (ACC_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }

    TIM_Waitms(ACC_DELAY_MS);
    time += ACC_DELAY_MS;
    
    if (time > ACC_TOTAL_TIME_MS)
      break;          

  }

  logInstClear();
  logInfoStop();

  return (xdone && ydone/* && zdone*/);
#endif

  int8_t x;
  int8_t y;
  int8_t z;

  testStartTime = mstick;

  acc_init();  

  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);
  acc_read(&x, &y, &z);



  if (x > -64 && x <=64 && y > -64 && y <=64 && z > -83 && z <= 83) {
    return TRUE;
  }
  
  sprintf(buff, "ACC: Failed  %d, %d, %d\r\n", x, y, z);
  LOG_INFO(buff);
  return FALSE;  
}

static uint32_t test_trimpot(void) 
{
#define TP_TOTAL_TIME_MS 20000
#define TP_DELAY_MS 250
#define MIN_OK 925

  uint32_t time = 0;
  uint32_t lasttime = 0;

  uint32_t x;

  uint32_t minx=0x3ff;//10-bit 
  uint32_t maxx=0;

  uint32_t xdone = FALSE;

  COLOR_T color = next_color();

//   // pinsel for AD0.2 on p0.25
// 	PINSEL_ConfigPin (0, 25, 1);

//  ADC_Init(LPC_ADC, 400000);

//	ADC_IntConfig(LPC_ADC, ADC_ADINTEN2, DISABLE);
//	ADC_ChannelCmd(LPC_ADC, ADC_CHANNEL_2, ENABLE);
  testStartTime = mstick;

  trimpot_init();

  logInst("Trimpot Test", 0, color);
  logInst("  Turn trimpot to endpoints", 1, color);

  sprintf(buff, "  %d sec remaining", (TP_TOTAL_TIME_MS-time)/1000);
  logInst(buff, 3, color);

  logInfoStart();

  while(1) {
  
		// Start conversion
		ADC_StartCmd(LPC_ADC0, ADC_START_NOW);

		//Wait conversion complete
		while (!(ADC_ChannelGetStatus(LPC_ADC0, ADC_CH_TRIMPOT, ADC_DATA_DONE)));

		x = ADC_ChannelGetData(LPC_ADC0, ADC_CH_TRIMPOT);

    if (x < minx)
      minx = x;

    if(x > maxx)
      maxx = x;

    /* check if values have changed with the minimum accepted value */
  
    if(!xdone && (maxx-minx) > MIN_OK){
      xdone = TRUE;
    }


    if (xdone)
      break;  

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (TP_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }

    TIM_Waitms(TP_DELAY_MS);
    time += TP_DELAY_MS;
    
    if (time > TP_TOTAL_TIME_MS)
      break;          

  }

  logInstClear();
  logInfoStop();

  return (xdone);
}

static uint32_t test_joystick(void) 
{
#define JOY_DELAY_MS 50
#define JOY_TOTAL_TIME_MS 10000
  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint8_t joy = 0;
  uint8_t pressed = 0;

  COLOR_T color = next_color();

  testStartTime = mstick;

  joystick_init();


  logInst("Move JOYSTICK", 0, color);
  logInst("  left, right, up", 1, color);
  logInst("  down and center", 2, color);

  logInfoStart();
  while(1) {
    joy = joystick_read();
    
    if ((joy & JOYSTICK_CENTER) != 0) {
      if ((pressed & JOYSTICK_CENTER) == 0)
        logInfo("Center");
      pressed |= JOYSTICK_CENTER;        
    }
    
    if ((joy & JOYSTICK_UP) != 0) {
      if ((pressed & JOYSTICK_UP) == 0)
        logInfo("Up");
      
      pressed |= JOYSTICK_UP;        
    }
    
    if ((joy & JOYSTICK_DOWN) != 0) {

      if ((pressed & JOYSTICK_DOWN) == 0)
        logInfo("Down");

      pressed |= JOYSTICK_DOWN;
    }
    
    if ((joy & JOYSTICK_LEFT) != 0) {
      if ((pressed & JOYSTICK_LEFT) == 0)
        logInfo("Left");

      pressed |= JOYSTICK_LEFT;        
    }
    
    if ((joy & JOYSTICK_RIGHT) != 0) {
      if ((pressed & JOYSTICK_RIGHT) == 0)
        logInfo("Right");

      pressed |= JOYSTICK_RIGHT;
    }
    
    if (pressed == (JOYSTICK_CENTER|JOYSTICK_LEFT|JOYSTICK_RIGHT|JOYSTICK_UP|JOYSTICK_DOWN ))
        break;

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (JOY_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }

    TIM_Waitms(JOY_DELAY_MS);
    time += JOY_DELAY_MS;
    
    if (time > JOY_TOTAL_TIME_MS)
      break;          

   
  }

  logInstClear();
  logInfoStop();
  return (pressed == (JOYSTICK_CENTER|JOYSTICK_LEFT|JOYSTICK_RIGHT|JOYSTICK_UP|JOYSTICK_DOWN ));
}

static uint32_t test_pca9532(void) 
{
#define LED_DELAY_MS 50
#define LED_TOTAL_TIME_MS 20000

  COLOR_T color = next_color();

  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint8_t ch = 0;
  uint32_t len = 0;
  uint16_t state = 0;
  uint16_t keyState = 0;

  testStartTime = mstick;

  pca9532_init();
  
  logInst("Test PCA9532", 0, color);
  logInst("Enter 'y' if all LEDs", 1, color);
  logInst("are turned on. Make sure ", 2, color);
  logInst("to also press 4 buttons ", 3, color);

  pca9532_setBlink0Period(151);
  pca9532_setBlink0Duty(50);

  pca9532_setBlink0Leds((LED1|LED2|LED3|LED4|LED5|LED6|LED7|LED8));  
  logInfoStart();  

    while(1) {
      if (ch != 'y' && ch != 'n') {

        len = console_receive(&ch, 1, NONE_BLOCKING);

        if (len == 0) {
          ch = 0;
        }
      }
            
      state = (pca9532_getLedState(FALSE));
    
      if ((state & KEY1) && (keyState & KEY1) == 0) {
        logInfo("Btn 1");
        keyState |= KEY1;
      }

      if ((state & KEY2) && (keyState & KEY2) == 0) {
        logInfo("Btn 2");
        keyState |= KEY2;
      }

      if ((state & KEY3) && (keyState & KEY3) == 0) {
        logInfo("Btn 3");
        keyState |= KEY3;
      }

      if ((state & KEY4) && (keyState & KEY4) == 0) {
        logInfo("Btn 4");
        keyState |= KEY4;
      }

      if ((ch == 'y' || ch == 'n') && keyState == (KEY1|KEY2|KEY3|KEY4)) {
        break;
      }
    

      if (time - lasttime >= 2000) {
        sprintf(buff, "  %d sec remaining", (LED_TOTAL_TIME_MS-time)/1000);
        logInst(buff, 5, color);
        lasttime = time;
      }
  
      TIM_Waitms(LED_DELAY_MS);
      time += LED_DELAY_MS;
      
      if (time > LED_TOTAL_TIME_MS)
        break;          

    }


    pca9532_setLeds(0, 0xFFFF);
  logInstClear();
  logInfoStop();
    return (ch == 'y');    
}

static uint32_t test_memreg(void) 
{
#define MEM_DELAY_MS 300
#define MEM_TOTAL_TIME_MS 10000

  COLOR_T color = next_color();

  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint8_t ch = 0;
  uint32_t len = 0;

  uint16_t memval = 0xffff;
  uint16_t* memreg = (uint16_t*)MEMREG_BASE;

  testStartTime = mstick;

  memreg_init();
  
  logInst("Test memmap LEDs", 0, color);
  logInst("Enter 'y' if all LEDs", 1, color);
  logInst("are turned on and then off", 2, color);

  
  while(1) {
    len = console_receive(&ch, 1, NONE_BLOCKING);

    if (len > 0 && (ch == 'y' || ch == 'n')) {
      break;
    }
          
    *memreg = memval;
    if (memval == 0xffff) {
      memval = 0x0000;
    } else {
      memval = 0xffff;
    }

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (MEM_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }

    TIM_Waitms(MEM_DELAY_MS);
    time += MEM_DELAY_MS;
    
    if (time > MEM_TOTAL_TIME_MS)
      break;          

  }

  *memreg = 0;

  logInstClear();
  return (ch == 'y');    
}


/* 
 * ---------------------------------------------
 * A U D I O
 * ---------------------------------------------
 */

#define AUDIO_TEST_DATA  audio
#define AUDIO_TEST_SIZE  audio_sz
extern const unsigned char audio[];
extern int audio_sz;
uint32_t buffer_offset = 0;
volatile uint32_t txDone = 0;

void I2S0_IRQHandler()
{
	uint32_t txlevel,i;
	txlevel = I2S_GetLevel(LPC_I2S0,I2S_TX_MODE);
	if(txlevel <= 4)
	{
		for(i = txlevel; i < 8; i++)
		{
			LPC_I2S0->TXFIFO = *(uint32_t *)(AUDIO_TEST_DATA + buffer_offset);
			buffer_offset +=4;
			if(buffer_offset >= AUDIO_TEST_SIZE)//reach the end of the song
			{
				buffer_offset = 0;
				txDone = 1;
				NVIC_DisableIRQ(I2S0_IRQn);
				I2S_Stop(LPC_I2S0, I2S_TX_MODE);
			}
		}
	}
}

void AudioTestInit( void ) 
{
  /* 		This code will set I2S in MASTER mode, 44100Hz, Mono,16bit
   * 		This example is used to test I2S transmit mode. There is an audio
   * 		data in audiodata.c in mono 44100Hz 16 bit, it will be send out to
   * 		I2S port. User must plug Ext I2S DAC IC to hear sound.
   */
	I2S_MODEConf_Type I2S_ClkConfig;
	I2S_CFG_Type I2S_ConfigStruct;
  int i;
  
	scu_pinmux(3,0, MD_PLN_FAST, FUNC2);// I2S0_TX_SCK
	scu_pinmux(0xc,12, MD_PLN_FAST, FUNC6);// I2S0_TX_SDA
	scu_pinmux(0xc,13, MD_PLN_FAST, FUNC6);// I2S0_TX_WS
  
  I2S_Init(LPC_I2S0);
  
	/* Audio Config*/
	I2S_ConfigStruct.wordwidth = I2S_WORDWIDTH_16;
	I2S_ConfigStruct.mono = I2S_MONO;
	I2S_ConfigStruct.stop = I2S_STOP_ENABLE;
	I2S_ConfigStruct.reset = I2S_RESET_ENABLE;
	I2S_ConfigStruct.ws_sel = I2S_MASTER_MODE;
	I2S_ConfigStruct.mute = I2S_MUTE_DISABLE;
	I2S_Config(LPC_I2S0,I2S_TX_MODE,&I2S_ConfigStruct);

	/* Clock Mode Config*/
	I2S_ClkConfig.clksel = I2S_CLKSEL_FRDCLK;
	I2S_ClkConfig.fpin = I2S_4PIN_DISABLE;
	I2S_ClkConfig.mcena = I2S_MCLK_DISABLE;
	I2S_ModeConfig(LPC_I2S0,&I2S_ClkConfig,I2S_TX_MODE);

	I2S_FreqConfig(LPC_I2S0, 44100, I2S_TX_MODE);

	I2S_Stop(LPC_I2S0, I2S_TX_MODE);

	/* Request interrupt when TX FIFO is half-full (depth is 4) */
	I2S_IRQConfig(LPC_I2S0,I2S_TX_MODE,4);
	I2S_IRQCmd(LPC_I2S0,I2S_TX_MODE,ENABLE);

  /* Important that I2S is started before UDA1380 is initialized as the
   * I2S clock must be running.
   */
	I2S_Start(LPC_I2S0);

	for(i = 0; i <0x1000000; i++);
  
	Uda1380_Init(100000, 44100);
  
	NVIC_ClearPendingIRQ(I2S0_IRQn);
	NVIC_EnableIRQ(I2S0_IRQn);  
}


static uint32_t test_audio(void) 
{
#define AD_DELAY_MS 300
#define AD_TOTAL_TIME_MS 50000

  COLOR_T color = next_color();

  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint8_t ch = 0;
  uint32_t len = 0;
  int numLoops = 3;
 

  testStartTime = mstick;

  AudioTestInit();

  logInst("Test audio", 0, color);
  logInst("Enter 'y' if audio is", 1, color);
  logInst("played in headphone and", 2, color);
  logInst("line out. Also test mic", 3, color);
  logInst("and line in", 4, color);

  
  while(1) {
    len = console_receive(&ch, 1, NONE_BLOCKING);

    if (len > 0 && (ch == 'y' || ch == 'n')) {
      break;
    }

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (AD_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }

    TIM_Waitms(AD_DELAY_MS);
    time += AD_DELAY_MS;
    
    if (time > AD_TOTAL_TIME_MS)
      break;      

    if (txDone) {
      if (--numLoops > 0) {
        // play one more time
        txDone = 0;
        NVIC_ClearPendingIRQ(I2S0_IRQn);
        I2S_Start(LPC_I2S0);
        NVIC_EnableIRQ(I2S0_IRQn);
      }
    }
  }

  logInstClear();
  return (ch == 'y');    
}

extern uint8_t inBuf[];
uint8_t mac[6] = {0x00, 0x06, 0x98, 0x01, 0x16, 0x36};
uint8_t localIP[4] = {192,168,5,241};

static uint32_t test_ethernet(void) 
{
#define ETH_TOTAL_TIME_MS 16000
#define ETH_DELAY_MS 5

  COLOR_T color = next_color();

  uint16_t len = 0;
  uint32_t numPing = 0;
  uint32_t lastNum = 0;
  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint32_t result = FALSE;
  uint8_t lbuf[30]; // = "192.168.5.236";

  char* pStr1 = NULL;
  char* pStr2 = NULL;

  FRESULT res;
  FIL file;
  UINT read = 0;

  int i = 0;

  testStartTime = mstick;

  logInst("Ethernet Test", 0, color);
  logInst("  Send at least 2 ping requests to ", 1, color);


  // ip.txt from sdcard

  res = f_open(&file, "ip.txt", FA_READ);
  if (res != FR_OK) {
    LOG_DEBUG("Failed to open file ip.txt \r\n");
    return FALSE;
  }

  res = f_read (&file, lbuf, 20, &read);			/* Read data from a file */
  if (res != FR_OK) {
    LOG_DEBUG("Failed to read ip.txt \r\n");
    return FALSE;
  }

  pStr1 = (char*)lbuf;

  for (i = 0; i < 3; i++)
  {
    pStr2 = strchr(pStr1, '.');

    if (pStr2 == NULL)
    {
      LOG_DEBUG("Ethernet: No valid IP in ip_address.txt\r\n");
      return FALSE;    
    }

    localIP[i] = strtol(pStr1, &pStr2, 10);
    pStr2++;
    pStr1 = pStr2;
  }
  
  pStr2 = pStr1+3;
  localIP[3] = strtol(pStr1, &pStr2, 10);

  sprintf(buff, "  IP: %d.%d.%d.%d", localIP[0],localIP[1],localIP[2],localIP[3]);
  logInst(buff, 2, color);

  if (ethIf_init(mac) != TRUE)
  {
    LOG_DEBUG("Ethernet: Initialization failed\r\n");
    return FALSE;
  }

  logInfoStart();

  pingReset();

  while(1)
  {

    len = ethIf_poll(inBuf, INBUF_LEN);

    if(len > 0)
    {
      ethInput(inBuf, len);

      numPing = pingGetNumReceived();
      if (lastNum != numPing)
      {
        sprintf(buff, "# Ping: %d", numPing);
        logInfo(buff);
        lastNum = numPing;  
      }

      if (numPing >= 2) 
      {
        result = TRUE;
        break;
      }

    }                                                                                     

    TIM_Waitms(ETH_DELAY_MS);
    time += ETH_DELAY_MS;

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (ETH_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }
      
    if (time > ETH_TOTAL_TIME_MS)
      break;          
    
  }

  logInstClear();
  logInfoStop();
  return result;    
}



USB_ClassInfo_MS_Host_t FlashDisk_MS_Interface =
	{
		.Config =
			{
				.DataINPipeNumber       = 1,
				.DataINPipeDoubleBank   = false,

				.DataOUTPipeNumber      = 2,
				.DataOUTPipeDoubleBank  = false,
			},
	};
  
//#include "usbhost_inc.h"

static int gUSBConnected = 0;
static int prevConnected = 0;
  
/** Event handler for the USB_DeviceAttached event. This indicates that a device has been attached to the host, and
 *  starts the library USB task to begin the enumeration and USB management process.
 */
void EVENT_USB_Host_DeviceAttached(void)
{
	gUSBConnected = 1;
}

/** Event handler for the USB_DeviceUnattached event. This indicates that a device has been removed from the host, and
 *  stops the library USB task management process.
 */
void EVENT_USB_Host_DeviceUnattached(void)
{
	gUSBConnected = 0;
}

  
static uint32_t test_usb(void) 
{  
#define USB_TOTAL_TIME_MS 20000
#define USB_DELAY_MS 5

  COLOR_T color = next_color();

  uint32_t time = 0;
  uint32_t lasttime = 0;
  uint8_t ch = 0;

  testStartTime = mstick;

  /* Hardware Initialization */
  USB_Init();

//   Host_Init();

  logInst("Test USB", 0, color);
  logInst("Insert and then remove USB stick", 1, color);
//   logInst("connect and disconnect", 2, color);  
//   logInst("from both ports", 3, color);  

  logInfoStart();
  while (1) {
//     len = console_receive(&ch, 1, NONE_BLOCKING);

//     if (len > 0 && (ch == 'y' || ch == 'n')) {
//       break;
//     }

    if (!prevConnected && gUSBConnected) {
      prevConnected = 1;
      logInfo("Connected");
    }
 
    if (prevConnected && !gUSBConnected) {
      prevConnected = 0;
      logInfo("Disconnected");
      
      // done
      ch = 'y';
      break;
    }

    TIM_Waitms(USB_DELAY_MS);
    time += USB_DELAY_MS;

    if (time - lasttime >= 2000) {
      sprintf(buff, "  %d sec remaining", (USB_TOTAL_TIME_MS-time)/1000);
      logInst(buff, 5, color);
      lasttime = time;
    }
      
    if (time > USB_TOTAL_TIME_MS)
      break;              
  } 
  logInstClear();
  logInfoStop();
  return (ch == 'y');    
}


/* SPIFI test is based on Examples\SPIFI\SPIFI_Test */
SPIFIobj obj;
SPIFI_RTNS * pSpifi;
SPIFIopers opers;
unsigned char ProgramData[PROG_SIZE];
static uint32_t test_spifi(void) 
{  
  uint32_t i;
  
  testStartTime = mstick;
  
#if 0
//  CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_SPIFI);
	/* Run SPIFI from PL160M, /2 */
	CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_CLKSRC_IDIVA);
	CGU_EnableEntity(CGU_CLKSRC_IDIVA, ENABLE);
	CGU_SetDIV(CGU_CLKSRC_IDIVA, 2);
	CGU_EntityConnect(CGU_CLKSRC_IDIVA, CGU_BASE_SPIFI);
	CGU_UpdateClock();
  #define SPIFI_MHZ  36  //(SystemCoreClock/2000000)
#else  
	/* set SPIFI clock */
	LPC_CGU->BASE_SPIFI_CLK = 1<<24 | 1<<11; /* IRC 12 MHz is good enough for us */
  #define SPIFI_MHZ  12
#endif  

	/* set up SPIFI I/O */
  scu_pinmux(3,3, PUP_DISABLE | PDN_DISABLE | SLEWRATE_FAST | INBUF_ENABLE | FILTER_DISABLE, FUNC3);// SPIFI_SCK
	/* IO pins */
  scu_pinmux(3,4, PUP_DISABLE | PDN_DISABLE | SLEWRATE_SLOW | INBUF_ENABLE | FILTER_DISABLE, FUNC3);// SPIFI_SIO3
  scu_pinmux(3,5, PUP_DISABLE | PDN_DISABLE | SLEWRATE_SLOW | INBUF_ENABLE | FILTER_DISABLE, FUNC3);// SPIFI_SIO2
  scu_pinmux(3,6, PUP_DISABLE | PDN_DISABLE | SLEWRATE_SLOW | INBUF_ENABLE | FILTER_DISABLE, FUNC3);// SPIFI_MISO
  scu_pinmux(3,7, PUP_DISABLE | PDN_DISABLE | SLEWRATE_SLOW | INBUF_ENABLE | FILTER_DISABLE, FUNC3);// SPIFI_MOSI
  scu_pinmux(3,8, PUP_DISABLE | PDN_DISABLE, FUNC3);// SPIFI_CS

#ifdef USE_SPIFI_LIB
  pSpifi = &spifi_table;
#else
  pSpifi = (SPIFI_RTNS *)(SPIFI_ROM_TABLE);
#endif
  
  /* Initialize SPIFI driver */
  if (pSpifi->spifi_init(&obj, 3, S_RCVCLK | S_FULLCLK, SPIFI_MHZ)) {
    return FALSE;
  }

  /* Erase Entire SPIFI Device if required */
//   for ( i = 0 ; i < obj.memSize / 4; i+=4 )
//   {
//     if ( *((uint32_t *)(obj.base+i)) != 0xFFFFFFFF )
//     {
//       opers.dest = (char *)(obj.base);
//       opers.length = obj.memSize;
//       opers.scratch = NULL;
//       opers.options = S_VERIFY_ERASE;
//       
//       /* Erase Device */
//       if (pSpifi->spifi_erase(&obj, &opers)) {
//         return FALSE;
//       }
//       break;
//     }
//   }
  
  /* Programming + verifying QSPI device */
	for(i=0;i<PROG_SIZE;i++) {
		ProgramData[i] = i%256;
  }
  
  opers.length = PROG_SIZE;
  opers.scratch = NULL;
  opers.protect = 0;
  opers.options = S_ERASE_AS_REQD;
  for ( i = 0 ; i < 4/*obj.memSize / PROG_SIZE*/; i++ )
  {
    /* Write */
    opers.dest = (char *)( obj.base + (i*PROG_SIZE) );
    if (pSpifi->spifi_program (&obj, (char *)ProgramData, &opers)) {
      return FALSE;
    }
    
    /* Verify */
    if (memcmp((void *)opers.dest,(void *)ProgramData,PROG_SIZE)) {
      return FALSE;
    }
  }
  
  return TRUE;
}


/******************************************************************************
 * Main method
 *****************************************************************************/



int main (void)
{
  SWIM_WINDOW_T win1;
  
  COLOR_T *fblog;
  
	TIM_TIMERCFG_Type timerCfg;

  uint32_t testsOk = TRUE;  
  uint32_t result  = TRUE;
  int32_t framebuf = (int32_t)(SDRAM_BASE+SDRAM_SIZE-FRAMEBUF_SZ);
  uint32_t demoMode = TRUE;
  uint8_t joy;
    
  SystemInit();
  
  joystick_init();
  joy = joystick_read();
  if (joy == JOYSTICK_DOWN)
  {
    // Run tests at maximum speed
    demoMode = FALSE;
    CGU_Init(BOARD_MAX_SPEED);
  }
  else if (joy == JOYSTICK_UP)
  {
    // Demo mode at full speed. This setting must be used
    // when running on a modified OemBaseBoard as those
    // cannot handle low frequencies.
    demoMode = TRUE;
    CGU_Init(BOARD_MAX_SPEED);
  }
  else
  {
    // Demo mode will be at the default 72MHz to be able to
    // run on unmodified OemBoards/OemBaseBoards. See the 
    // User's Manual for information about the SDRAM speed
    // limits and the modifications that can be made to 
    // work around the problem
    demoMode = TRUE;
    CGU_Init(72000000);
  }
  
  CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_LCD);

  RGU_SoftReset(RGU_SIG_ADC0);
  RGU_SoftReset(RGU_SIG_DAC);
  RGU_SoftReset(RGU_SIG_LCD);
  RGU_SoftReset(RGU_SIG_DMA);
  RGU_SoftReset(RGU_SIG_EMC);
  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  RGU_SoftReset(RGU_SIG_I2C0);
  RGU_SoftReset(RGU_SIG_SSP0);
  RGU_SoftReset(RGU_SIG_SDIO);
  RGU_SoftReset(RGU_SIG_USB0);
  RGU_SoftReset(RGU_SIG_SPIFI);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  /* Initialize I2C */
  I2C_Init(I2C_PORT, 100000);
  I2C_Cmd(I2C_PORT, ENABLE);

  /* Request systicks every 1 ms */
  SysTick_Config(SystemCoreClock/1000);

  // See if test or demo should be executed
  if (demoMode) {
    demo_main();
  }

  PrintGreeting();

  LOG_INFO("Starting tests\r\n");

//   /* Make sure it is the LPC4357 that is being tested */
//   result = CPUID_IS_4357;
//   logResult("CPUID", result);
//   testsOk &= result;

  /* SDRAM has been tested by ea_test_part1, so only initialize */
  result = sdram_init();
  logResult("SDRAM", result);
  testsOk &= result;

  /* only use display if SDRAM is ok */
  if (result) {
    memset((void*)(framebuf), 0x00, FRAMEBUF_SZ);
  
    truly_lcd_open(framebuf);
  
    fblog = (COLOR_T*)framebuf;

    /* Create a SWIM window */
    swim_window_open(&win1, DISPLAY_WIDTH, DISPLAY_HEIGHT, fblog, 0, 0, 
      DISPLAY_WIDTH-1, DISPLAY_HEIGHT-1, 1, WHITE, BLACK, BLACK);
    
    win = &win1;
    swim_set_font(win, (FONT_T*)&font_x5x7);

    lcd_ok = TRUE;
  }

  result = test_eepromOem();       //works, enough with ID
  logResult("EEPROM 1", result);
  testsOk &= result;

  result = test_nand();  //works
  logResult("NAND", result);
  testsOk &= result;

  result = test_rtc();  // works
  logResult("RTC", result);
  testsOk &= result;

  // ------- B A S K O R T -----

  result = test_nor();   //works
  logResult("NOR", result);
  testsOk &= result;

  result = test_sdcard();      // works
  logResult("SDCARD", result);
  testsOk &= result;

//   result = test_temp();        //works, skip
//   logResult("TEMP", result);
//   testsOk &= result;

  result = test_ethernet();  //works
  logResult("Ethernet", result);
  testsOk &= result;

//   result = test_acc();         //works, skip
//   logResult("ACC", result);
//   testsOk &= result;

  result = test_trimpot();     //works
  logResult("Trimpot", result);
  testsOk &= result;

  result = test_joystick();       //works
  logResult("Joystick", result);
  testsOk &= result;

//    result = test_pca9532();      //works, but tested by ea_test_part1
//    logResult("PCA9532", result);
//    testsOk &= result;

  result = test_memreg();      //works
  logResult("LEDs (m)", result);
  testsOk &= result;

  result = test_usb();
  logResult("USB", result);
  testsOk &= result;

  result = test_audio();
  logResult("Audio", result);
  testsOk &= result;

  result = test_spifi(); //works, perhaps only read ID
  logResult("SPIFI", result);
  testsOk &= result;

  /* Print the final result */
  if (testsOk) {

    swim_set_fill_color(&win1, GREEN);
    swim_put_box(&win1, 0, logy, DISPLAY_WIDTH, DISPLAY_HEIGHT-1); 
   
    LOG_INFO("\r\n\r\nTests: PASSED\r\n");
  }
  else {

    swim_set_fill_color(&win1, RED);
    swim_put_box(&win1, 0, logy, DISPLAY_WIDTH, DISPLAY_HEIGHT-1); 

    LOG_INFO("\r\n\r\nTests: FAILED\r\n");
  }


  while(1) {
    //LOG_INFO("\r\n\r\nTests: FAILED\r\n");
  }
}


