/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_i2c.h"
#include "lpc43xx_i2s.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_ssp.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_uart.h"

#include "board.h"
#include "uda1380.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define I2C_PORT (LPC_I2C0)

#define DATA_SIZE			15658
#define BUFFER_SIZE			1048 * 4

/******************************************************************************
 * Local variables
 *****************************************************************************/

/* Buffer for printing */
static char buff[512];

extern unsigned char audio[];
uint8_t tx_buffer[BUFFER_SIZE];
uint32_t data_offset, buffer_offset,remain_data;
uint32_t tx_offset = 0;
Bool Tx_EndofBuf_F = FALSE;
volatile uint32_t txDone = 1;


/******************************************************************************
 * Forward declarations
 *****************************************************************************/
void I2S_Callback(void);
void I2S0_IRQHandler(void);
void Buffer_Init(void);


/******************************************************************************
 * External Functions
 *****************************************************************************/

/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		I2S IRQ Handler, call to send data to transmit buffer
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2S0_IRQHandler()
{
	uint32_t txlevel,i;
	txlevel = I2S_GetLevel(LPC_I2S0,I2S_TX_MODE);
	if(txlevel <= 4)
	{
		for(i=0;i<8-txlevel;i++)
		{
			LPC_I2S0->TXFIFO = *(uint32_t *)(tx_buffer + buffer_offset);
			tx_offset +=4;
			buffer_offset +=4;
			//call I2S_Callback() function when reach each half of tx_buffer
			if((buffer_offset == BUFFER_SIZE/2)||(buffer_offset == BUFFER_SIZE))
				I2S_Callback();
			if(tx_offset >= DATA_SIZE)//reach the end of buffer
			{
				tx_offset = 0;
				Buffer_Init();
				txDone = 1;
				NVIC_DisableIRQ(I2S0_IRQn);
				I2S_Stop(LPC_I2S0, I2S_TX_MODE);
			}
		}
	}
}

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n***************************************************\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("* I2S audio test program for LPC4357 OEM Board... *\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                  *\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("*                           " __DATE__ ", " __TIME__ " *\r\n");
  console_sendString("***************************************************\r\n");
}

/*********************************************************************//**
 * @brief		Initialize transmit buffer
 * @param[in]	none
 * @return 		None
 **********************************************************************/
void Buffer_Init(void)
{
	memcpy(tx_buffer, audio, BUFFER_SIZE);
	buffer_offset = 0;
	data_offset = BUFFER_SIZE;
	remain_data = DATA_SIZE - BUFFER_SIZE;
}

/*********************************************************************//**
 * @brief		I2S callback function, will be call when I2S send a half of
 				buffer complete
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2S_Callback(void)
{
	if(remain_data >=BUFFER_SIZE/2)
	{
		if(buffer_offset == BUFFER_SIZE)
		{
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer + BUFFER_SIZE/2, audio+data_offset, BUFFER_SIZE/2);
			buffer_offset = 0;
		}
		else
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer, audio+data_offset, BUFFER_SIZE/2);
			data_offset += BUFFER_SIZE/2;
			remain_data -= BUFFER_SIZE/2;
	}
	else // reach the last copy
	{
		if(buffer_offset == BUFFER_SIZE)
		{
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer + BUFFER_SIZE/2, audio+data_offset, remain_data);
			buffer_offset = 0;
		}
		else
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer, audio+data_offset, remain_data);

	}
}

/******************************************************************************
 * Main method
 *****************************************************************************/



int main (void)
{
  /* 		This code will set I2S in MASTER mode, 44100Hz, Mono,16bit
   * 		This example is used to test I2S transmit mode. There is an audio
   * 		data in audiodata.c in mono 44100Hz 16 bit, it will be send out to
   * 		I2S port. User must plug Ext I2S DAC IC to hear sound.
   */
	I2S_MODEConf_Type I2S_ClkConfig;
	I2S_CFG_Type I2S_ConfigStruct;
	TIM_TIMERCFG_Type timerCfg;
  int i;

  SystemInit();
  CGU_Init(BOARD_MAX_SPEED);
  Buffer_Init();

  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  RGU_SoftReset(RGU_SIG_I2C0);
  RGU_SoftReset(RGU_SIG_I2S);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  /* Initialize I2C */
  I2C_Init(I2C_PORT, 100000);
  I2C_Cmd(I2C_PORT, ENABLE);

  PrintGreeting();

  /* Initialize I2S */
	scu_pinmux(3,0, MD_PLN_FAST, FUNC2);// I2S0_TX_SCK
	scu_pinmux(0xc,12, MD_PLN_FAST, FUNC6);// I2S0_TX_SDA
	scu_pinmux(0xc,13, MD_PLN_FAST, FUNC6);// I2S0_TX_WS
  
	I2S_Init(LPC_I2S0);

	/* setup:
	 * 		- wordwidth: 16 bits
	 * 		- mono mode
	 * 		- master mode for I2S_TX
	 * 		- Frequency = 44.1 kHz
	 */

	/* Audio Config*/
	I2S_ConfigStruct.wordwidth = I2S_WORDWIDTH_16;
	I2S_ConfigStruct.mono = I2S_MONO;
	I2S_ConfigStruct.stop = I2S_STOP_ENABLE;
	I2S_ConfigStruct.reset = I2S_RESET_ENABLE;
	I2S_ConfigStruct.ws_sel = I2S_MASTER_MODE;
	I2S_ConfigStruct.mute = I2S_MUTE_DISABLE;
	I2S_Config(LPC_I2S0,I2S_TX_MODE,&I2S_ConfigStruct);

	/* Clock Mode Config*/
	I2S_ClkConfig.clksel = I2S_CLKSEL_FRDCLK;
	I2S_ClkConfig.fpin = I2S_4PIN_DISABLE;
	I2S_ClkConfig.mcena = I2S_MCLK_DISABLE;
	I2S_ModeConfig(LPC_I2S0,&I2S_ClkConfig,I2S_TX_MODE);

	I2S_FreqConfig(LPC_I2S0, 44100, I2S_TX_MODE);

	I2S_Stop(LPC_I2S0, I2S_TX_MODE);

	/* TX FIFO depth is 4 */
	I2S_IRQConfig(LPC_I2S0,I2S_TX_MODE,4);
	I2S_IRQCmd(LPC_I2S0,I2S_TX_MODE,ENABLE);

  /* Important that I2S is started before UDA1380 is initialized as the
   * I2S clock must be running.
   */
	I2S_Start(LPC_I2S0);

	for(i = 0; i <0x1000000; i++);
  
	Uda1380_Init(100000, 44100);
  
	NVIC_ClearPendingIRQ(I2S0_IRQn);
	NVIC_EnableIRQ(I2S0_IRQn);  
  
	/* repeat the sound in a loop */
	while(1) {

		if(txDone) {
			txDone = 0;

			NVIC_ClearPendingIRQ(I2S0_IRQn);

			I2S_Start(LPC_I2S0);

			NVIC_EnableIRQ(I2S0_IRQn);
		}
	}  
}


