/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_uart.h"

#include "board.h"
#include "diskio.h"
#include "ff.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/* Maximum size of buffer for benchmarking */
#define MAX_BLOCK_SIZE  8192

/* Tests if the file object represents a directory */
#define IS_DIR(_finfo) ((_finfo).fattrib & AM_DIR)

/* Tests if the file object represents the current director (aka ".") */
#define IS_DOT_DIR(_finfo) (((_finfo).fname[0]=='.') && ((_finfo).fname[1]=='\0'))

/* Tests if the file object represents the parent director (aka "..") */
#define IS_DOTDOT_DIR(_finfo) (((_finfo).fname[0]=='.') && ((_finfo).fname[1]=='.') && ((_finfo).fname[2]=='\0'))

/* The file to benchmark. Should be at least one MB to get meassurable differences in read speed */
#define BENCH_FILE  "/bench/bigfile.bin"

/******************************************************************************
 * Local Variables
 *****************************************************************************/

/* Buffer for benchmarking */
static char benchbuff[MAX_BLOCK_SIZE];

/* Buffer for printing */
static char buff[512];

/* Absolute path used when listing file system recursively */
static char dirnamebuff[512];

/* Simple benchmarking of time */
static volatile uint32_t currentTime = 0;

static FATFS lFatfs[1];
static FILINFO Finfo;

#if _USE_LFN
char Lfname[_MAX_LFN+1];
#endif

/******************************************************************************
 * Forward declarations
 *****************************************************************************/


/******************************************************************************
 * External Functions
 *****************************************************************************/

void SysTick_Handler(void)
{
  currentTime++;
}

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n***************************************************\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("* FATFS/SDIO test program for LPC4357 OEM Board...*\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                  *\r\n");
  console_sendString("*                                                 *\r\n");
  sprintf(buff, "*                           %s, %s *\r\n", __DATE__, __TIME__);
  console_sendString((uint8_t*)buff);
  console_sendString("***************************************************\r\n");
}

FRESULT listRecursive(char* dirname)
{
  DIR dir;
  FRESULT res;
	int pos = strlen(dirname);
  
#if _USE_LFN
  memset(Lfname, 0, _MAX_LFN+1);
#endif

  res = f_opendir(&dir, dirname);
  if (res) {
    sprintf(buff, "Failed to open %s: %d \r\n", dirname, res);
    console_sendString((uint8_t*)buff);
    return res;
  }

	sprintf(buff, "Listing content of \"%s\"\r\n", dirname);
  console_sendString((uint8_t*)buff);
  for(;res == FR_OK;) {
    res = f_readdir(&dir, &Finfo);
    if ((res != FR_OK) || !Finfo.fname[0]) break;
    
		if (IS_DIR(Finfo)) 
		{
			if (IS_DOT_DIR(Finfo) || IS_DOTDOT_DIR(Finfo))
			{
				// Ignore . and .. 
			}
			else
			{
				if (dirname[1] == '\0')
				{
					sprintf(dirnamebuff, "%s%s", dirname, Finfo.fname);
				}
				else
				{
					sprintf(dirnamebuff, "%s/%s", dirname, Finfo.fname);
				}
				res = listRecursive(dirnamebuff);
				
				// Restore dirnamebuff 
				dirnamebuff[pos] = '\0';
			}
		}
		else
		{
			console_sendString((uint8_t*)&(Finfo.fname[0]));
#if _USE_LFN
			console_sendString("\t");
			console_sendString((uint8_t*)&(Finfo.lfname[0]));
#endif
			console_sendString((uint8_t*)"\r\n");
		}
  }
	return res;
}

FRESULT benchmarkLargeFile(char* fname, uint32_t blockSize)
{
  FRESULT res;
  FIL file;
  UINT numRead = 0;
	uint32_t duration;

 	sprintf(buff, "Benchmarking %s in blocks of %d bytes:\r\n", fname, blockSize);
  console_sendString((uint8_t*)buff);
	if (blockSize > MAX_BLOCK_SIZE)
	{
		sprintf(buff, "Bad block size (%d). Must be < %d. Aborting!\r\n", blockSize, MAX_BLOCK_SIZE);
		console_sendString((uint8_t*)buff);
		return FR_OK;
	}
	currentTime = 0;
  res = f_open (&file, fname, FA_READ);
  if (res != FR_OK) {
    sprintf(buff, "Failed to open: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return res;
  }

	while ((res == FR_OK) && (numRead < file.fsize))
	{
		uint32_t tmp = 0;
		res = f_read (&file, benchbuff, MIN(file.fsize, blockSize), &tmp);			/* Read data from a file */
		if ((res == FR_OK) && (tmp > 0))
		{
			numRead += tmp;
		}
	}
	duration = currentTime;
	if (res == FR_OK)
	{
		sprintf(buff, "  Reading %d bytes took %d ms => ca %d Kb/s\r\n", file.fsize, duration*10, file.fsize/(duration*10));
	}
	else
	{
		sprintf(buff, "  Failed after %d bytes and %d ms with error %d\r\n", file.fsize, duration*10, res);
	}
  console_sendString((uint8_t*)buff);
	f_close(&file);
	return res;
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
  FRESULT res;
  FIL file;
  UINT numRead = 0;
  UINT numWritten = 0;
  TIM_TIMERCFG_Type timerCfg;
  uint32_t size;

  SystemInit();
  CGU_Init(BOARD_MAX_SPEED);

  RGU_SoftReset(RGU_SIG_DMA);
  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  RGU_SoftReset(RGU_SIG_SDIO);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  PrintGreeting();

  SysTick_Config(SystemCoreClock/100);
  
  console_sendString((uint8_t*)"FatFS Demo\r\n");

#if _USE_LFN
	Finfo.lfname = Lfname;
	Finfo.lfsize = sizeof(Lfname);
#endif

  res = f_mount(0, &lFatfs[0]);
  if (res != FR_OK) {
    sprintf(buff, "Failed to mount 0: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }
  
  res = listRecursive("/");
  
  size = MAX_BLOCK_SIZE;
  while (size >= 512)
  {
    benchmarkLargeFile(BENCH_FILE, size);
    size = size >> 1;
  }

  console_sendString((uint8_t*)"\r\nOpening a file\r\n");
  res = f_open (&file, "ip.txt", FA_READ);
  if (res) {
    sprintf(buff, "Failed to open ip.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  console_sendString((uint8_t*)"Reading content of the file\r\n");
  res =  f_read (&file, buff, 100, &numRead);			/* Read data from a file */
  if (res || numRead <= 0) {
    sprintf(buff, "Failed to read ip.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  buff[numRead] = '\0';
  console_sendString((uint8_t*)buff);

  console_sendString((uint8_t*)"\r\nClosing file\r\n");
  res =  f_close (&file);
  if (res) {
    sprintf(buff, "Failed to close ip.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  console_sendString((uint8_t*)"\r\nOpening a file for writing\r\n");
  res = f_open (&file, "new.txt", (FA_CREATE_ALWAYS|FA_READ|FA_WRITE));
  if (res) {
    sprintf(buff, "Failed to open new.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  console_sendString((uint8_t*)"Writing  to the file\r\n");
  res = f_write (&file, "Data written to file\r\n", 22, &numWritten);
  if (res) {
    sprintf(buff, "Failed to write to new.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  console_sendString((uint8_t*)"Closing file\r\n");
  res =  f_close (&file);
  if (res) {
    sprintf(buff, "Failed to close new.txt: %d \r\n", res);
    console_sendString((uint8_t*)buff);
    return 0;
  }

  console_sendString((uint8_t*)"\r\nDONE\r\n");
  
  while(1);
}


