/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_i2c.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_ssp.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_uart.h"

#include "board.h"

#include "bsp.h"
#include "USB.h"
#include <ctype.h>
#include <stdio.h>



/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/


/******************************************************************************
 * Local variables
 *****************************************************************************/

/* Buffer for printing */
static char buff[512];

/** nxpUSBlib Mass Storage Class driver interface configuration and state information. This structure is
 *  passed to all Mass Storage Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_MS_Host_t FlashDisk_MS_Interface =
	{
		.Config =
			{
				.DataINPipeNumber       = 1,
				.DataINPipeDoubleBank   = false,

				.DataOUTPipeNumber      = 2,
				.DataOUTPipeDoubleBank  = false,
			},
	};

/******************************************************************************
 * External Functions
 *****************************************************************************/


/******************************************************************************
 * Function Forward Declarations
 *****************************************************************************/

void SetupHardware(void);
void MassStorageHost_Task(void);

void EVENT_USB_Host_HostError(const uint8_t ErrorCode);
void EVENT_USB_Host_DeviceAttached(void);
void EVENT_USB_Host_DeviceUnattached(void);
void EVENT_USB_Host_DeviceEnumerationFailed(const uint8_t ErrorCode,
                                            const uint8_t SubErrorCode);
void EVENT_USB_Host_DeviceEnumerationComplete(void);


/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n***************************************************\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("* USB Host MassStorage for LPC4357 OEM Board...   *\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                  *\r\n");
  console_sendString("*                                                 *\r\n");
  sprintf(buff, "*                           %s, %s *\r\n", __DATE__, __TIME__);
  console_sendString((uint8_t*)buff);
  console_sendString("***************************************************\r\n");
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void)
{
	bsp_init();

	/* Hardware Initialization */
	USB_Init();
}

/** Task to manage an enumerated USB Mass Storage device once connected, to print out
 *  data from the device.
 */
void MassStorageHost_Task(void)
{
	if (USB_HostState != HOST_STATE_Configured)
	  return;

	console_sendString("Waiting until ready...\r\n");

	for (;;)
	{
		uint8_t ErrorCode = MS_Host_TestUnitReady(&FlashDisk_MS_Interface, 0);

		if (!(ErrorCode))
		  break;

		/* Check if an error other than a logical command error (device busy) received */
		if (ErrorCode != MS_ERROR_LOGICAL_CMD_FAILED)
		{
			console_sendString("Error waiting for device to be ready.\r\n");
			USB_Host_SetDeviceConfiguration(0);
			return;
		}
	}

	console_sendString("Retrieving Capacity...\r\n");

	SCSI_Capacity_t DiskCapacity;
	if (MS_Host_ReadDeviceCapacity(&FlashDisk_MS_Interface, 0, &DiskCapacity))
	{
		console_sendString("Error retrieving device capacity.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	printf_P(PSTR("%lu blocks of %lu bytes.\r\n"), DiskCapacity.Blocks, DiskCapacity.BlockSize);

	//uint8_t BlockBuffer[DiskCapacity.BlockSize];
	uint8_t BlockBuffer[512];

	if (MS_Host_ReadDeviceBlocks(&FlashDisk_MS_Interface, 0, 0x00000000, 1, DiskCapacity.BlockSize, BlockBuffer))
	{
		console_sendString("Error reading device block.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	console_sendString("\r\nContents of first block:\r\n");

	for (uint16_t Chunk = 0; Chunk < (DiskCapacity.BlockSize >> 4); Chunk++)
	{
		uint8_t* ChunkPtr = &BlockBuffer[Chunk << 4];

		/* Print out the 16 bytes of the chunk in HEX format */
		for (uint8_t ByteOffset = 0; ByteOffset < (1 << 4); ByteOffset++)
		{
			char CurrByte = *(ChunkPtr + ByteOffset);
			printf_P(PSTR("%.2X "), CurrByte);
		}

		console_sendString("    ");

		/* Print out the 16 bytes of the chunk in ASCII format */
		for (uint8_t ByteOffset = 0; ByteOffset < (1 << 4); ByteOffset++)
		{
			char CurrByte = *(ChunkPtr + ByteOffset);
			putchar(isprint(CurrByte) ? CurrByte : '.');
		}

		console_sendString("\r\n");
	}

	USB_Host_SetDeviceConfiguration(0);
}

/** Event handler for the USB_DeviceAttached event. This indicates that a device has been attached to the host, and
 *  starts the library USB task to begin the enumeration and USB management process.
 */
void EVENT_USB_Host_DeviceAttached(void)
{
	console_sendString("Device Attached.\r\n");
}

/** Event handler for the USB_DeviceUnattached event. This indicates that a device has been removed from the host, and
 *  stops the library USB task management process.
 */
void EVENT_USB_Host_DeviceUnattached(void)
{
	console_sendString("\r\nDevice Unattached.\r\n");
}

/** Event handler for the USB_DeviceEnumerationComplete event. This indicates that a device has been successfully
 *  enumerated by the host and is now ready to be used by the application.
 */
void EVENT_USB_Host_DeviceEnumerationComplete(void)
{
	uint16_t ConfigDescriptorSize;
	uint8_t  ConfigDescriptorData[512];

	if (USB_Host_GetDeviceConfigDescriptor(1, &ConfigDescriptorSize, ConfigDescriptorData,
	                                       sizeof(ConfigDescriptorData)) != HOST_GETCONFIG_Successful)
	{
		console_sendString("Error Retrieving Configuration Descriptor.\r\n");
		return;
	}

	if (MS_Host_ConfigurePipes(&FlashDisk_MS_Interface,
	                           ConfigDescriptorSize, ConfigDescriptorData) != MS_ENUMERROR_NoError)
	{
		console_sendString("Attached Device Not a Valid Mass Storage Device.\r\n");
		return;
	}

	if (USB_Host_SetDeviceConfiguration(1) != HOST_SENDCONTROL_Successful)
	{
		console_sendString("Error Setting Device Configuration.\r\n");
		return;
	}

	uint8_t MaxLUNIndex;
	if (MS_Host_GetMaxLUN(&FlashDisk_MS_Interface, &MaxLUNIndex))
	{
		console_sendString("Error retrieving max LUN index.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

  sprintf(buff, "Total LUNs: %d - Using first LUN in device.\r\n", (MaxLUNIndex + 1));
  console_sendString((uint8_t*)buff);	

	if (MS_Host_ResetMSInterface(&FlashDisk_MS_Interface))
	{
		console_sendString("Error resetting Mass Storage interface.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	SCSI_Request_Sense_Response_t SenseData;
	if (MS_Host_RequestSense(&FlashDisk_MS_Interface, 0, &SenseData) != 0)
	{
		console_sendString("Error retrieving device sense.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	if (MS_Host_PreventAllowMediumRemoval(&FlashDisk_MS_Interface, 0, true))
	{
		console_sendString("Error setting Prevent Device Removal bit.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	SCSI_Inquiry_Response_t InquiryData;
	if (MS_Host_GetInquiryData(&FlashDisk_MS_Interface, 0, &InquiryData))
	{
		console_sendString("Error retrieving device Inquiry data.\r\n");
		USB_Host_SetDeviceConfiguration(0);
		return;
	}

	printf_P(PSTR("Vendor \"%.8s\", Product \"%.16s\"\r\n"), InquiryData.VendorID, InquiryData.ProductID);

	console_sendString("Mass Storage Device Enumerated.\r\n");
}

/** Event handler for the USB_HostError event. This indicates that a hardware error occurred while in host mode. */
void EVENT_USB_Host_HostError(const uint8_t ErrorCode)
{
	USB_Disable();

  sprintf(buff, "Host Mode Error\r\n"
	              " -- Error Code %d\r\n", ErrorCode);
  console_sendString((uint8_t*)buff);

	for(;;);
}

/** Event handler for the USB_DeviceEnumerationFailed event. This indicates that a problem occurred while
 *  enumerating an attached USB device.
 */
void EVENT_USB_Host_DeviceEnumerationFailed(const uint8_t ErrorCode,
                                            const uint8_t SubErrorCode)
{
  sprintf(buff, "Dev Enum Error\r\n"
	              " -- Error Code %d\r\n"
	              " -- Sub Error Code %d\r\n"
	              " -- In State %d\r\n", ErrorCode, SubErrorCode, USB_HostState);
  console_sendString((uint8_t*)buff);
}

/******************************************************************************
 * Main method
 *****************************************************************************/



int main (void)
{
	TIM_TIMERCFG_Type timerCfg;
  uint8_t old;

  SystemInit();
  CGU_Init(BOARD_MAX_SPEED);

  RGU_SoftReset(RGU_SIG_LCD);
  RGU_SoftReset(RGU_SIG_DMA);
  RGU_SoftReset(RGU_SIG_EMC);
  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  RGU_SoftReset(RGU_SIG_I2C0);
  RGU_SoftReset(RGU_SIG_SSP0);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  PrintGreeting();

  sprintf(buff, "Using USB port %d, SystemCoreClock = %dMHz\r\n", USB_PORT, SystemCoreClock/1000000);
  console_sendString((uint8_t*)buff);

  SetupHardware();
  
  old = 0xff;
  while (1)
  {
    if (old != USB_HostState) {
      old = USB_HostState;
      sprintf(buff, "USB_HostState is now %u\r\n", USB_HostState);
      console_sendString((uint8_t*)buff);
    }
		MassStorageHost_Task();
		
		MS_Host_USBTask(&FlashDisk_MS_Interface);
		USB_USBTask();
  }
}


